@@ -75,6 +75,13 @@ namespace Ogre {
         @par
             A BillboardSet can be created using the SceneManager::createBillboardSet method. They can also be used internally
             by other classes to create effects.
+		@note
+			Billboard bounds are only automatically calculated when you create them.
+			If you modify the position of a billboard you may need to call 
+			_updateBounds if the billboard moves outside the original bounds. 
+			Similarly, the bounds do no shrink when you remove a billboard, 
+			if you want them to call _updateBounds, but note this requires a
+			potentially expensive examination of every billboard in the set.
     */
     class _OgreExport BillboardSet : public MovableObject, public Renderable
     {
@@ -444,6 +451,12 @@ namespace Ogre {
         void injectBillboard(const Billboard& bb);
         /** Finish defining billboards. */
         void endBillboards(void);
+		/** Set the bounds of the BillboardSet.
+		@remarks
+			You may need to call this if you're injecting billboards manually, 
+			and you're relying on the BillboardSet to determine culling.
+		*/
+		void setBounds(const AxisAlignedBox& box, Real radius);
 
 
         /** Overridden from MovableObject
