@@ -159,6 +159,10 @@ namespace Ogre {
         bool mDisplaySkeleton;
         /// Flag indicating whether hardware skinning is supported by this entities materials
         bool mHardwareSkinning;
+        /// Counter indicating number of requests for software skinning.
+        int mSoftwareSkinningRequests;
+        /// Counter indicating number of requests for software skinned normals.
+        int mSoftwareSkinningNormalsRequests;
         /// Flag indicating whether we have a vertex program in use on any of our subentities
         bool mVertexProgramInUse;
 
@@ -334,6 +338,9 @@ namespace Ogre {
         */
         AnimationState* getAnimationState(const String& name);
         /** For entities based on animated meshes, gets the AnimationState objects for all animations.
+        @returns
+            In case the entity is animated, this functions returns the pointer to a AnimationStateSet 
+            containing all animations of the entries. If the entity is not animated, it returns 0.
         @remarks
             You animate an entity by updating the animation state objects. Each of these represents the
             current state of each animation available to the entity. The AnimationState objects are
@@ -345,6 +352,24 @@ namespace Ogre {
         */
         void setDisplaySkeleton(bool display);
 
+        /** Returns whether or not the entity is currently displaying its skeleton.
+         */
+        bool getDisplaySkeleton(void) const;
+
+
+        /** Gets a pointer to the entity representing the numbered manual level of detail.
+        @remarks
+            The zero-based index never includes the original entity, unlike
+			Mesh::getLodLevel.
+        */
+        Entity* getManualLodLevel(size_t index) const;
+
+        /** Returns the number of manual levels of detail that this entity supports.
+        @remarks
+            This number never includes the original entity, it is difference
+            with Mesh::getNumLodLevels.
+        */
+        size_t getNumManualLodLevels(void) const;
 
 		/** Sets a level-of-detail bias for the mesh detail of this entity.
 		@remarks
@@ -405,6 +430,10 @@ namespace Ogre {
         /** Sets the rendering detail of this entire entity (solid, wireframe etc) */
         void setRenderDetail(SceneDetailLevel renderDetail);
 
+		/** Sets whether the rendering detail of this entire entity may be 
+			overridden by the camera detail settings. 
+		*/
+		void setRenderDetailOverrideable(bool renderDetailOverrideable);
 		/** Attaches another object to a certain bone of the skeleton which this entity uses.
         @remarks
             This method can be used to attach another object to an animated part of this entity,
@@ -436,6 +465,11 @@ namespace Ogre {
         ChildObjectListIterator getAttachedObjectIterator(void);
         /** @see MovableObject::getBoundingRadius */
 		Real getBoundingRadius(void) const;
+		/** @copy MovableObject::getWorldBoundingBox */
+		const AxisAlignedBox& getWorldBoundingBox(bool derive = false) const;
+		/** @copy MovableObject::getWorldBoundingSphere */
+		const Sphere& getWorldBoundingSphere(bool derive = false) const;
+
         /** If set to true, this forces normals of this entity to be normalised
             dynamically by the hardware.
         @remarks
@@ -475,7 +509,52 @@ namespace Ogre {
             assigned to this entity have vertex programs assigned, and all those
             vertex programs must support 'include_skeletal_animation true'.
         */
-        bool isHardwareSkinningEnabled(void) { return mHardwareSkinning; }
+        bool isHardwareSkinningEnabled(void) const { return mHardwareSkinning; }
+        /** Returns the number of requests that have been made for software skinning
+        @remarks
+            If non-zero then software skinning will be performed in updateAnimation
+            regardless of the current setting of isHardwareSkinningEnabled.
+            Requests for software skinning are made by calling the 
+            addSoftwareSkinningRequest() method.
+        */
+        int getSoftwareSkinningRequests(void) const { return mSoftwareSkinningRequests; }
+        /** Returns the number of requests that have been made for software skinning of normals
+        @remarks
+            If non-zero, and getSoftwareSkinningRequests() also returns non-zero,
+            then software skinning of normals will be performed in updateAnimation
+            regardless of the current setting of isHardwareSkinningEnabled.
+            Currently it is not possible to force software skinning of only normals.
+            Consequently this value is always less than or equal to that returned
+            by getSoftwareSkinningRequests().
+            Requests for software skinning of normals are made by calling the 
+            addSoftwareSkinningRequest() method with 'true' as the parameter.
+        */
+        int getSoftwareSkinningNormalsRequests(void) const { return mSoftwareSkinningNormalsRequests; }
+        /** Add a request for software skinning
+        @remarks
+            Tells the entity to perform skinning calculations for skeletal
+            animations in software, regardless of the current setting of 
+            isHardwareSkinningEnabled().  Software skinning will be performed
+            any time one or more requests have been made.  If 'normalsAlso' is
+            'true', then the entity will also do software blending on normal
+            vectors, in addition to positions. This advanced method useful for 
+            situations in which access to actual mesh vertices is required,
+            such as accurate collision detection or certain advanced shading 
+            techniques.  When software skinning is no longer needed, 
+            the caller of this method should always remove the request by calling
+            removeSoftwareSkinningRequest(), passing the same value for 
+            'normalsAlso'.
+        */
+        void addSoftwareSkinningRequest(bool normalsAlso);
+        /** Removes a request for software skinning
+        @remarks
+            Calling this decrements the entity's internal counter of the number
+            of requests for software skinning.  If the counter is already zero
+            then calling this method throws an exception.  The 'normalsAlso' 
+            flag if set to 'true' will also decrement the internal counter of
+            number of requests for software skinning of normals.
+        */
+        void removeSoftwareSkinningRequest(bool normalsAlso);
 
         /** Overridden from MovableObject */
         void _notifyAttached(Node* parent, bool isTagPoint = false);
@@ -503,6 +582,18 @@ namespace Ogre {
         */
         inline const EntitySet* getSkeletonInstanceSharingSet() const { return mSharedSkeletonEntities; }
 
+		/** Updates the internal animation state set to include the latest 
+			available animations from the attached skeleton.
+		@remarks
+			Use this method if you manually add animations to a skeleton, or have
+			linked the skeleton to another for animation purposes since creating 
+			this entity.
+		@note
+			If you have called getAnimationState prior to calling this method, 
+			the pointers will still remain valid.
+		*/
+		void refreshAvailableAnimationState(void);
+
 		/** Advanced method to perform all the updates required for an animated entity.
 		@remarks
 			You don't normally need to call this, but it's here incase you wish
