@@ -119,6 +119,7 @@ namespace Ogre {
             ACT_CUSTOM,
             /// The current camera's position in world space
             ACT_CAMERA_POSITION,
+            ACT_TIME,
 			/** Single float value, which repeats itself based on given as
 			parameter "cycle time". Equivalent to RenderMonkey's "Time0_X".
 			*/
@@ -188,7 +189,7 @@ namespace Ogre {
 			Equivalent to RenderMonkey's "ViewDirection".
 			*/
 			ACT_VIEW_DIRECTION,
-			/** This variable provides the view size vector (world space).
+			/** This variable provides the view side vector (world space).
 			Equivalent to RenderMonkey's "ViewSideVector".
 			*/
 			ACT_VIEW_SIDE_VECTOR,
@@ -278,7 +279,7 @@ namespace Ogre {
  
         };
         /** Structure recording the use of an automatic parameter. */
-        class AutoConstantEntry
+        class _OgrePrivate AutoConstantEntry
         {
         public:
             /// The type of parameter
@@ -348,6 +349,14 @@ namespace Ogre {
 		@param vec The value to set
 		*/
 		void setConstant(size_t index, const Vector4& vec);
+		/** Sets a single floating-point parameter to the program.
+		@note This is actually equivalent to calling 
+		setConstant(index Vector4(val, 0, 0, 0)) since all constants are 4D.
+		@param index The constant index at which to place the parameter (each constant is
+		a 4D float)
+		@param val The value to set
+		*/
+		void setConstant(size_t index, Real val);
 		/** Sets a 4-element floating-point parameter to the program via Vector3.
 		@param index The constant index at which to place the parameter (each constant is
             a 4D float).
