@@ -98,11 +98,18 @@ namespace Ogre {
         typedef std::map<unsigned short, Technique*> BestTechniqueList;
         BestTechniqueList mBestTechniqueList;
 
-        /// Does this material require compilation?
-        bool mCompilationRequired;
         LodDistanceList mLodDistances;
         bool mReceiveShadows;
 		bool mTransparencyCastsShadows;
+        /// Does this material require compilation?
+        bool mCompilationRequired;
+
+        /** Fixup the best technique list guarantees no gaps inside.
+        @remarks
+            Iterate over the best technique list, looking for gaps and filling them in
+            guarantees we've got a sequential list with entries in all indexes
+        */
+        void fixupBestTechniqueList(void);
 
 		/** Overridden from Resource.
 		*/
@@ -241,7 +248,7 @@ namespace Ogre {
             Compiling a material involves determining which Techniques are supported on the
             card on which OGRE is currently running, and for fixed-function Passes within those
             Techniques, splitting the passes down where they contain more TextureUnitState 
-            instances than the curren card has texture units.
+            instances than the current card has texture units.
         @par
             This process is automatically done when the Material is loaded, but may be
             repeated if you make some procedural changes.
@@ -553,6 +560,14 @@ namespace Ogre {
             Resource::touch();
         }
 
+	    /** Gets the compilation status of the material.
+        @return True if the material needs recompilation.
+        */
+	    bool getCompilationRequired() const
+	    {
+		    return mCompilationRequired;
+	    }
+
 
     };
 
