@@ -107,6 +107,27 @@ namespace Ogre
             m[3][3] = m33;
         }
 
+        /** Creates a standard 4x4 transformation matrix with a zero translation part from a rotation/scaling 3x3 matrix.
+         */
+
+        inline Matrix4(const Matrix3& m3x3)
+        {
+          operator=(IDENTITY);
+          operator=(m3x3);
+        }
+
+        /** Creates a standard 4x4 transformation matrix with a zero translation part from a rotation/scaling Quaternion.
+         */
+        
+        inline Matrix4(const Quaternion& rot)
+        {
+          Matrix3 m3x3;
+          rot.ToRotationMatrix(m3x3);
+          operator=(IDENTITY);
+          operator=(m3x3);
+        }
+        
+
         inline Real* operator [] ( size_t iRow )
         {
             assert( iRow < 4 );
@@ -185,12 +206,13 @@ namespace Ogre
         inline Plane operator * (const Plane& p) const
         {
             Plane ret;
+			Matrix4 invTrans = inverse().transpose();
             ret.normal.x =
-                m[0][0] * p.normal.x + m[0][1] * p.normal.y + m[0][2] * p.normal.z;
+                invTrans[0][0] * p.normal.x + invTrans[0][1] * p.normal.y + invTrans[0][2] * p.normal.z;
             ret.normal.y = 
-                m[1][0] * p.normal.x + m[1][1] * p.normal.y + m[1][2] * p.normal.z;
+                invTrans[1][0] * p.normal.x + invTrans[1][1] * p.normal.y + invTrans[1][2] * p.normal.z;
             ret.normal.z = 
-                m[2][0] * p.normal.x + m[2][1] * p.normal.y + m[2][2] * p.normal.z;
+                invTrans[2][0] * p.normal.x + invTrans[2][1] * p.normal.y + invTrans[2][2] * p.normal.z;
             Vector3 pt = p.normal * -p.d;
             pt = *this * pt;
             ret.d = - pt.dotProduct(ret.normal);
@@ -270,7 +292,7 @@ namespace Ogre
 
         /** Tests 2 matrices for inequality.
         */
-        inline bool operator != ( Matrix4& m2 ) const
+        inline bool operator != ( const Matrix4& m2 ) const
         {
             if( 
                 m[0][0] != m2.m[0][0] || m[0][1] != m2.m[0][1] || m[0][2] != m2.m[0][2] || m[0][3] != m2.m[0][3] ||
@@ -312,6 +334,14 @@ namespace Ogre
             m[2][3] = v.z;
         }
 
+        /** Extracts the translation transformation part of the matrix.
+         */
+        inline Vector3 getTrans() const
+        {
+          return Vector3(m[0][3], m[1][3], m[2][3]);
+        }
+        
+
         /** Builds a translation matrix
         */
         inline void makeTrans( const Vector3& v )
@@ -415,6 +445,15 @@ namespace Ogre
 
         }
 
+        /** Extracts the rotation / scaling part as a quaternion from the Matrix.
+         */
+        inline Quaternion extractQuaternion() const
+        {
+          Matrix3 m3x3;
+          extract3x3Matrix(m3x3);
+          return Quaternion(m3x3);
+        }
+
         static const Matrix4 ZERO;
         static const Matrix4 IDENTITY;
         /** Useful little matrix which takes 2D clipspace {-1, 1} to {0,1}
