@@ -41,6 +41,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreConfigOptionMap.h"
 #include "OgreGpuProgram.h"
 #include "OgrePlane.h"
+#include "OgreIteratorWrappers.h"
 
 namespace Ogre
 {
@@ -294,13 +295,14 @@ namespace Ogre
 				Key: "externalWindowHandle" [API specific]
 				Description: External window handle, for embedding the OGRE context
 				Values: positive integer for W32 (HWND handle)
-				        posint:posint:posint for GLX (display:screen:windowHandle)
+				        poslong:posint:poslong (display*:screen:windowHandle) or
+				        poslong:posint:poslong:poslong (display*:screen:windowHandle:XVisualInfo*) for GLX
 				Default: 0 (None)
 				**
 				Key: "parentWindowHandle" [API specific]
 				Description: Parent window handle, for embedding the OGRE context
 				Values: positive integer for W32 (HWND handle)
-				        posint:posint:posint for GLX (display:screen:windowHandle)
+				        poslong:posint:poslong for GLX (display*:screen:windowHandle)
 				Default: 0 (None)
 				**
 				Key: "FSAA"
@@ -317,6 +319,17 @@ namespace Ogre
 				Description: Synchronize buffer swaps to vsync
 				Values: true, false
 				Default: 0
+				** 
+				Key: "border" 
+				Description: The type of window border (in windowed mode)
+				Values: none, fixed, resize
+				Default: resize
+				**
+				Key: "outerDimensions" 
+				Description: Whether the width/height is expressed as the size of the 
+				outer window, rather than the content area
+				Values: true, false
+				Default: false 
         */
 		virtual RenderWindow* createRenderWindow(const String &name, unsigned int width, unsigned int height, 
 			bool fullScreen, const NameValuePairList *miscParams = 0) = 0;
@@ -380,6 +393,13 @@ namespace Ogre
         */
         virtual RenderTarget * detachRenderTarget( const String &name );
 
+		/// Iterator over RenderTargets
+		typedef MapIterator<Ogre::RenderTargetMap> RenderTargetIterator;
+
+		/** Returns a specialised MapIterator over all render targets attached to the RenderSystem. */
+		virtual RenderTargetIterator getRenderTargetIterator(void) {
+			return RenderTargetIterator( mRenderTargets.begin(), mRenderTargets.end() );
+		}
         /** Returns a description of an error code.
         */
         virtual String getErrorDescription(long errorNumber) const = 0;
@@ -934,6 +954,48 @@ namespace Ogre
         @see Renderable::useIdentityView, Renderable::useIdentityProjection
         */
         virtual Real getMaximumDepthInputValue(void) = 0;
+
+		/** Defines a listener on the custom events that this render system 
+			can raise.
+		@see RenderSystem::addListener
+		*/
+		class _OgreExport Listener
+		{
+		public:
+			Listener() {}
+			virtual ~Listener() {}
+
+			/** A rendersystem-specific event occurred.
+			@param eventName The name of the event which has occurred
+			@param parameters A list of parameters that may belong to this event,
+				may be null if there are no parameters
+			*/
+			virtual void eventOccurred(const String& eventName, 
+				const NameValuePairList* parameters = 0) = 0;
+		};
+		/** Adds a listener to the custom events that this render system can raise.
+		@remarks
+			Some render systems have quite specific, internally generated events 
+			that the application may wish to be notified of. Many applications
+			don't have to worry about these events, and can just trust OGRE to 
+			handle them, but if you want to know, you can add a listener here.
+		@par
+			Events are raised very generically by string name. Perhaps the most 
+			common example of a render system specific event is the loss and 
+			restoration of a device in DirectX; which OGRE deals with, but you 
+			may wish to know when it happens. 
+		@see RenderSystem::getRenderSystemEvents
+		*/
+		virtual void addListener(Listener* l);
+		/** Remove a listener to the custom events that this render system can raise.
+		*/
+		virtual void removeListener(Listener* l);
+
+		/** Gets a list of the rendersystem specific events that this rendersystem
+			can raise.
+		@see RenderSystem::addListener
+		*/
+		virtual const StringVector& getRenderSystemEvents(void) const { return mEventNames; }
     protected:
 
 
@@ -972,6 +1034,15 @@ namespace Ogre
 
         bool mInvertVertexWinding;
 
+		/// List of names of events this rendersystem may raise
+		StringVector mEventNames;
+
+		/// Internal method for firing a rendersystem event
+		virtual void fireEvent(const String& name, const NameValuePairList* params = 0);
+
+		typedef std::list<Listener*> ListenerList;
+		ListenerList mEventListeners;
+
     };
 }
 
