@@ -63,6 +63,8 @@ namespace Ogre {
     class _OgreExport ResourceGroupListener
     {
     public:
+        virtual ~ResourceGroupListener() {}
+
 		/** This event is fired when a resource group begins parsing scripts.
 		@param groupName The name of the group 
 		@param scriptCount The number of scripts which will be parsed
@@ -451,10 +453,12 @@ namespace Ogre {
             a number of resource types: 
             <ul>
             <li>Font</li>
+            <li>GpuProgram</li>
+            <li>HighLevelGpuProgram</li>
             <li>Material</li>
             <li>Mesh</li>
-            <li>Overlay</li>
             <li>Skeleton</li>
+            <li>Texture</li>
             </ul>
             .. but more can be added by plugin ResourceManager classes.
         @param groupName The name of the group to which it will belong.
@@ -661,6 +665,20 @@ namespace Ogre {
         */
         void _notifyWorldGeometryStageEnded(void);
 
+		/** Get a list of the currently defined resource groups. 
+		@note This method intentionally returns a copy rather than a reference in
+			order to avoid any contention issues in multithreaded applications.
+		@returns A copy of list of currently defined groups.
+		*/
+		StringVector getResourceGroups(void);
+		/** Get the list of resource declarations for the specified group name. 
+		@note This method intentionally returns a copy rather than a reference in
+			order to avoid any contention issues in multithreaded applications.
+		@param groupName The name of the group
+		@returns A copy of list of currently defined resources.
+		*/
+		ResourceDeclarationList getResourceDeclarationList(const String& groupName);
+
 		/** Override standard Singleton retrieval.
         @remarks
         Why do we do this? Well, it's because the Singleton
