@@ -44,6 +44,9 @@ namespace Ogre {
 
 #define OGRE_MAX_NUM_BONES 256
 
+	
+	struct LinkedSkeletonAnimationSource;
+
     /** A collection of Bone objects used to animate a skinned mesh.
     @remarks
         Skeletal animation works by having a collection of 'bones' which are 
@@ -80,8 +83,7 @@ namespace Ogre {
         /** Constructor, don't call directly, use SkeletonManager.
         @remarks
             On creation, a Skeleton has a no bones, you should create them and link
-            them together appropriately. Unless you state otherwise by attaching it to 
-            a higher bone, the first bone you create is deemed to be the root bone.
+            them together appropriately. 
         */
         Skeleton(ResourceManager* creator, const String& name, ResourceHandle handle,
             const String& group, bool isManual = false, ManualResourceLoader* loader = 0);
@@ -90,9 +92,9 @@ namespace Ogre {
 
         /** Creates a brand new Bone owned by this Skeleton. 
         @remarks
-            This method creates an unattached new Bone for this skeleton. Unless this is to
-            be the root bone (there must only be one of these), you must
-            attach it to another Bone in the skeleton using addChild for it to be any use. 
+            This method creates an unattached new Bone for this skeleton.
+			Unless this is to be a root bone (there may be more than one of 
+			these), you must attach it to another Bone in the skeleton using addChild for it to be any use.
             For this reason you will likely be better off creating child bones using the
             Bone::createChild method instead, once you have created the root bone. 
         @par
@@ -106,11 +108,10 @@ namespace Ogre {
         /** Creates a brand new Bone owned by this Skeleton. 
         @remarks
             This method creates an unattached new Bone for this skeleton and assigns it a 
-            specific handle. Unless this is to
-            be the root bone (there must only be one of these), you must
-            attach it to another Bone in the skeleton using addChild for it to be any use. 
+            specific handle. Unless this is to be a root bone (there may be more than one of 
+			these), you must attach it to another Bone in the skeleton using addChild for it to be any use. 
             For this reason you will likely be better off creating child bones using the
-            Bone::createChild method instead, once you have created the root bone. 
+            Bone::createChild method instead, once you have created a root bone. 
         @param handle The handle to give to this new bone - must be unique within this skeleton. 
             You should also ensure that all bone handles are eventually contiguous (this is to simplify
             their compilation into an indexed array of transformation matrices). For this reason
@@ -122,9 +123,8 @@ namespace Ogre {
         /** Creates a brand new Bone owned by this Skeleton. 
         @remarks
             This method creates an unattached new Bone for this skeleton and assigns it a 
-            specific name. Unless this is to
-            be the root bone (there must only be one of these), you must
-            attach it to another Bone in the skeleton using addChild for it to be any use. 
+            specific name.Unless this is to be a root bone (there may be more than one of 
+			these), you must attach it to another Bone in the skeleton using addChild for it to be any use.
             For this reason you will likely be better off creating child bones using the
             Bone::createChild method instead, once you have created the root bone. 
         @param name The name to give to this new bone - must be unique within this skeleton. 
@@ -138,9 +138,8 @@ namespace Ogre {
         /** Creates a brand new Bone owned by this Skeleton. 
         @remarks
             This method creates an unattached new Bone for this skeleton and assigns it a 
-            specific name and handle. Unless this is to
-            be the root bone (there must only be one of these), you must
-            attach it to another Bone in the skeleton using addChild for it to be any use. 
+            specific name and handle. Unless this is to be a root bone (there may be more than one of 
+			these), you must attach it to another Bone in the skeleton using addChild for it to be any use.
             For this reason you will likely be better off creating child bones using the
             Bone::createChild method instead, once you have created the root bone. 
         @param name The name to give to this new bone - must be unique within this skeleton. 
@@ -199,8 +198,16 @@ namespace Ogre {
         */
         virtual Animation* createAnimation(const String& name, Real length);
 
-        /** Returns the named Animation object. */
-        virtual Animation* getAnimation(const String& name) const;
+        /** Returns the named Animation object. 
+		@remarks
+			Will pick up animations in linked skeletons 
+			(@see addLinkedSkeletonAnimationSource). 
+		@param name The name of the animation
+		@param linker Optional pointer to a pointer to the linked skeleton animation
+			where this is coming from.
+		*/
+        virtual Animation* getAnimation(const String& name, 
+			const LinkedSkeletonAnimationSource** linker = 0) const;
 
         /** Removes an Animation from this skeleton. */
         virtual void removeAnimation(const String& name);
@@ -222,13 +229,19 @@ namespace Ogre {
         virtual const AnimationStateSet& getAnimationState(void) const;
         
 
-        /** Initialise an animation set suitable for use with this mesh. 
+        /** Initialise an animation set suitable for use with this skeleton. 
         @remarks
             Only recommended for use inside the engine, not by applications.
         */
         virtual void _initAnimationState(AnimationStateSet* animSet);
 
-        /** Populates the passed in array with the bone matrices based on the current position.
+		/** Refresh an animation set suitable for use with this skeleton. 
+		@remarks
+			Only recommended for use inside the engine, not by applications.
+		*/
+		virtual void _refreshAnimationState(AnimationStateSet* animSet);
+
+		/** Populates the passed in array with the bone matrices based on the current position.
         @remarks
             Internal use only. The array pointed to by the passed in pointer must
             be at least as large as the number of bones.
@@ -239,7 +252,11 @@ namespace Ogre {
         /** Gets the number of animations on this skeleton. */
         virtual unsigned short getNumAnimations(void) const;
 
-        /** Gets a single animation by index. */
+        /** Gets a single animation by index. 
+		@remarks
+			Will NOT pick up animations in linked skeletons 
+			(@see addLinkedSkeletonAnimationSource).
+		*/
         virtual Animation* getAnimation(unsigned short index) const;
 
 
@@ -251,8 +268,58 @@ namespace Ogre {
         /// Updates all the derived transforms in the skeleton
         virtual void _updateTransforms(void);
 
+		/** Optimise all of this skeleton's animations.
+		@see Animation::optimise
+		*/
+		virtual void optimiseAllAnimations(void);
 
-    protected:
+		/** Allows you to use the animations from another Skeleton object to animate
+			this skeleton.
+		@remarks
+			If you have skeletons of identical structure (that means identically
+			named bones with identical handles, and with the same hierarchy), but
+			slightly different proportions or binding poses, you can re-use animations
+			from one in the other. Because animations are actually stored as
+			changes to bones from their bind positions, it's possible to use the
+			same animation data for different skeletons, provided the skeletal
+			structure matches and the 'deltas' stored in the keyframes apply
+			equally well to the other skeletons bind position (so they must be
+			roughly similar, but don't have to be identical). You can use the 
+			'scale' option to adjust the translation and scale keyframes where
+			there are large differences in size between the skeletons.
+		@note
+			This method takes a skeleton name, rather than a more specific 
+			animation name, for two reasons; firstly it allows some validation 
+			of compatibility of skeletal structure, and secondly skeletons are
+			the unit of loading. Linking a skeleton to another in this way means
+			that the linkee will be prevented from being destroyed until the 
+			linker is destroyed.
+
+			You cannot set up cyclic relationships, e.g. SkeletonA uses SkeletonB's
+			animations, and SkeletonB uses SkeletonA's animations. This is because
+			it would set up a circular dependency which would prevent proper 
+			unloading - make one of the skeletons the 'master' in this case.
+		@param skelName Name of the skeleton to link animations from. This 
+			skeleton will be loaded immediately if this skeleton is already 
+			loaded, otherwise it will be loaded when this skeleton is.
+		@param scale A scale factor to apply to translation and scaling elements
+			of the keyframes in the other skeleton when applying the animations
+			to this one. Compensates for skeleton size differences.
+		*/
+		virtual void addLinkedSkeletonAnimationSource(const String& skelName, 
+			Real scale = 1.0f);
+		/// Remove all links to other skeletons for the purposes of sharing animation
+		virtual void removeAllLinkedSkeletonAnimationSources(void);
+		
+		typedef std::vector<LinkedSkeletonAnimationSource> 
+			LinkedSkeletonAnimSourceList;
+		typedef ConstVectorIterator<LinkedSkeletonAnimSourceList> 
+			LinkedSkeletonAnimSourceIterator;
+		/// Get an iterator over the linked skeletons used as animation sources
+		virtual LinkedSkeletonAnimSourceIterator 
+			getLinkedSkeletonAnimationSourceIterator(void) const;
+
+	protected:
 		SkeletonAnimationBlendMode mBlendState;
         /// Storage of bones, indexed by bone handle
         BoneList mBoneList;
@@ -271,10 +338,12 @@ namespace Ogre {
         typedef std::map<String, Animation*> AnimationList;
         AnimationList mAnimationsList;
 
-
         /// Saved version of last animation
         AnimationStateSet mLastAnimationState;
 
+		/// List of references to other skeletons to use animations from 
+		mutable LinkedSkeletonAnimSourceList mLinkedSkeletonAnimSourceList;
+
         /** Internal method which parses the bones to derive the root bone. 
         @remarks
             Must be const because called in getRootBone but mRootBone is mutable
@@ -341,6 +410,18 @@ namespace Ogre {
         }
     };
 
+	/// Link to another skeleton to share animations
+	struct LinkedSkeletonAnimationSource
+	{
+		String skeletonName;
+		SkeletonPtr pSkeleton;
+		Real scale;
+		LinkedSkeletonAnimationSource(const String& skelName, Real scl)
+			: skeletonName(skelName), scale(scl) {}
+			LinkedSkeletonAnimationSource(const String& skelName, Real scl, 
+				SkeletonPtr skelPtr)
+				: skeletonName(skelName), pSkeleton(skelPtr), scale(scl) {}
+	};
 }
 
 
