@@ -416,7 +416,7 @@ namespace Ogre {
         @note
         Has no effect in the programmable pipeline.
         */
-        const Matrix4& getTextureTransform(void);
+        const Matrix4& getTextureTransform(void) const;
 
         /** Sets the translation offset of the texture, ie scrolls the texture.
         @remarks
@@ -861,8 +861,6 @@ namespace Ogre {
 
 	
 protected:
-        Pass* mParent;
-
         // State
 #define OGRE_MAX_TEXTURE_FRAMES 32
 
@@ -874,7 +872,6 @@ protected:
 
         /// Duration of animation in seconds
         Real mAnimDuration;            
-        Controller<Real>* mAnimController;
         bool mCubic; // is this a series of 6 2D textures to make up a cube?
 		
         TextureType mTextureType; 
@@ -890,11 +887,11 @@ protected:
         LayerBlendModeEx alphaBlendMode;
         bool mIsBlank;
 
-        bool mRecalcTexMatrix;
+        mutable bool mRecalcTexMatrix;
         Real mUMod, mVMod;
         Real mUScale, mVScale;
         Radian mRotate;
-        Matrix4 mTexModMatrix;
+        mutable Matrix4 mTexModMatrix;
 
         // Animation, will be set up as Controllers
         Real mUScrollAnim, mVScrollAnim;
@@ -923,11 +920,18 @@ protected:
         EffectMap mEffects;
         //-----------------------------------------------------------------------------
 
+        //-----------------------------------------------------------------------------
+        // Pointer members (those that can't be copied using memcpy), and MUST
+        // preserving even if assign from others
+        //
+        Pass* mParent;
+        Controller<Real>* mAnimController;
+        //-----------------------------------------------------------------------------
 
 
         /** Internal method for calculating texture matrix.
         */
-        void recalcTextureMatrix(void);
+        void recalcTextureMatrix(void) const;
 
         /** Internal method for creating animation controller.
         */
