@@ -50,6 +50,7 @@ namespace Ogre {
         mIndexData(0),
         mCullIndividual( false ),
         mBillboardType(BBT_POINT),
+        mCommonDirection(Ogre::Vector3::UNIT_Z),
         mBuffersCreated(false),
         mPoolSize(0),
         mExternalData(false)
@@ -74,6 +75,7 @@ namespace Ogre {
         mIndexData(0),
         mCullIndividual( false ),
         mBillboardType(BBT_POINT),
+        mCommonDirection(Ogre::Vector3::UNIT_Z),
         mBuffersCreated(false),
         mPoolSize(poolSize),
         mExternalData(externalData)
@@ -130,10 +132,23 @@ namespace Ogre {
 
         newBill->setPosition(position);
         newBill->setColour(colour);
+        newBill->mDirection = Vector3::ZERO;
+        newBill->setRotation(Radian(0));
+        newBill->resetDimensions();
         newBill->_notifyOwner(this);
 
-        _updateBounds();
+		// Merge into bounds
+		Real adjust = std::max(mDefaultWidth, mDefaultHeight);
+        Vector3 vecAdjust(adjust, adjust, adjust);
+		Vector3 newMin = position - vecAdjust;
+		Vector3 newMax = position + vecAdjust;
+		
+        mAABB.merge(newMin);
+        mAABB.merge(newMax);
 
+		Real sqlen = std::max(newMin.squaredLength(), newMax.squaredLength());
+		mBoundingRadius = std::max(mBoundingRadius, Math::Sqrt(sqlen));
+		
         return newBill;
     }
 
@@ -410,6 +425,12 @@ namespace Ogre {
     {
         mMainBuf->unlock();
     }
+	//-----------------------------------------------------------------------
+	void BillboardSet::setBounds(const AxisAlignedBox& box, Real radius)
+	{
+		mAABB = box;
+		mBoundingRadius = radius;
+	}
     //-----------------------------------------------------------------------
     void BillboardSet::_updateBounds(void)
     {
