@@ -52,9 +52,11 @@ namespace Ogre {
         mFullBoundingBox = new AxisAlignedBox;
         mNormaliseNormals = false;
         mFrameBonesLastUpdated = new unsigned long;
-        *mFrameBonesLastUpdated = 0;
-        mFrameAnimationLastUpdated = 0;
+		*mFrameBonesLastUpdated = std::numeric_limits<unsigned long>::max();
+        mFrameAnimationLastUpdated = std::numeric_limits<unsigned long>::max();
         mHardwareSkinning = false;
+        mSoftwareSkinningRequests = 0;
+        mSoftwareSkinningNormalsRequests = 0;
         mSkeletonInstance = 0;
     }
     //-----------------------------------------------------------------------
@@ -66,6 +68,8 @@ namespace Ogre {
     {
         mFullBoundingBox = new AxisAlignedBox;
         mHardwareSkinning = false;
+        mSoftwareSkinningRequests = 0;
+        mSoftwareSkinningNormalsRequests = 0;
         mSharedBlendedVertexData = NULL;
 
         // Is mesh skeletally animated?
@@ -101,12 +105,11 @@ namespace Ogre {
 
 
         // Initialise the AnimationState, if Mesh has animation
-
         if (hasSkeleton())
         {
             mAnimationState = new AnimationStateSet();
             mFrameBonesLastUpdated = new unsigned long;
-            *mFrameBonesLastUpdated = 0;
+            *mFrameBonesLastUpdated = std::numeric_limits<unsigned long>::max();
             mesh->_initAnimationState(mAnimationState);
             mNumBoneMatrices = mSkeletonInstance->getNumBones();
             mBoneMatrices = new Matrix4[mNumBoneMatrices];
@@ -135,7 +138,7 @@ namespace Ogre {
         mMinMaterialLodIndex = 99;
 
 
-        mFrameAnimationLastUpdated = 0;
+        mFrameAnimationLastUpdated = std::numeric_limits<unsigned long>::max();
 
         // Do we have a mesh where edge lists are not going to be available?
         if (!mesh->isEdgeListBuilt() && !mesh->getAutoBuildEdgeLists())
@@ -194,6 +197,8 @@ namespace Ogre {
                 delete mFrameBonesLastUpdated;
             }
         }
+
+		delete mSharedBlendedVertexData;
     }
     //-----------------------------------------------------------------------
     MeshPtr& Entity::getMesh(void)
@@ -335,6 +340,37 @@ namespace Ogre {
 
         return full_aa_box;
     }
+	//-----------------------------------------------------------------------
+	const AxisAlignedBox& Entity::getWorldBoundingBox(bool derive) const
+	{
+		if (derive)
+		{
+			// derive child bounding boxes
+			ChildObjectList::const_iterator child_itr = mChildObjectList.begin();
+			ChildObjectList::const_iterator child_itr_end = mChildObjectList.end();
+			for( ; child_itr != child_itr_end; child_itr++)
+			{
+				child_itr->second->getWorldBoundingBox(true);
+			}
+		}
+		return MovableObject::getWorldBoundingBox(derive);
+	}
+	//-----------------------------------------------------------------------
+	const Sphere& Entity::getWorldBoundingSphere(bool derive) const
+	{
+		if (derive)
+		{
+			// derive child bounding boxes
+			ChildObjectList::const_iterator child_itr = mChildObjectList.begin();
+			ChildObjectList::const_iterator child_itr_end = mChildObjectList.end();
+			for( ; child_itr != child_itr_end; child_itr++)
+			{
+				child_itr->second->getWorldBoundingSphere(true);
+			}
+		}
+		return MovableObject::getWorldBoundingSphere(derive);
+
+	}
     //-----------------------------------------------------------------------
     void Entity::_updateRenderQueue(RenderQueue* queue)
     {
@@ -452,14 +488,17 @@ namespace Ogre {
 
             // Software blend?
             bool hwSkinning = isHardwareSkinningEnabled();
-            if (!hwSkinning ||
+            bool forcedSwSkinning = getSoftwareSkinningRequests()>0;
+            bool forcedNormals = getSoftwareSkinningNormalsRequests()>0;
+            if (!hwSkinning || forcedSwSkinning ||
                 root._getCurrentSceneManager()->getShadowTechnique() == SHADOWTYPE_STENCIL_ADDITIVE ||
                 root._getCurrentSceneManager()->getShadowTechnique() == SHADOWTYPE_STENCIL_MODULATIVE)
             {
+                bool blendNormals = !hwSkinning || forcedNormals;
+
                 // Ok, we need to do a software blend
                 // Blend normals in s/w only if we're not using h/w skinning,
                 // since shadows only require positions
-                bool blendNormals = !hwSkinning;
                 // Firstly, check out working vertex buffers
                 if (mSharedBlendedVertexData)
                 {
@@ -489,9 +528,7 @@ namespace Ogre {
                     }
 
                 }
-
             }
-
             // Trigger update of bounding box if necessary
             if (!mChildObjectList.empty())
                 mParentNode->needUpdate();
@@ -574,6 +611,7 @@ namespace Ogre {
             // replacement world matrices
             unsigned short i;
             Matrix4 worldXform = _getParentNodeFullTransform();
+            assert (mNumBoneMatrices==mSkeletonInstance->getNumBones());
             mNumBoneMatrices = mSkeletonInstance->getNumBones();
 
             for (i = 0; i < mNumBoneMatrices; ++i)
@@ -590,6 +628,23 @@ namespace Ogre {
         mDisplaySkeleton = display;
     }
     //-----------------------------------------------------------------------
+    bool Entity::getDisplaySkeleton(void) const
+    {
+        return mDisplaySkeleton;
+    }
+    //-----------------------------------------------------------------------
+    Entity* Entity::getManualLodLevel(size_t index) const
+    {
+        assert(index < mLodEntityList.size());
+
+        return mLodEntityList[index];
+    }
+    //-----------------------------------------------------------------------
+    size_t Entity::getNumManualLodLevels(void) const
+    {
+        return mLodEntityList.size();
+    }
+    //-----------------------------------------------------------------------
     void Entity::setMeshLodBias(Real factor, ushort maxDetailIndex, ushort minDetailIndex)
     {
         assert(factor > 0.0f && "Bias factor must be > 0!");
@@ -636,6 +691,17 @@ namespace Ogre {
             (*i)->setRenderDetail(renderDetail); 
         } 
     }
+    //-----------------------------------------------------------------------
+    void Entity::setRenderDetailOverrideable(bool renderDetailOverrideable) 
+    {
+        SubEntityList::iterator i, iend;
+        iend = mSubEntityList.end();
+
+        for( i = mSubEntityList.begin(); i != iend; ++i ) 
+        { 
+            (*i)->setRenderDetailOverrideable(renderDetailOverrideable);
+        } 
+    }
 
     //-----------------------------------------------------------------------
     void Entity::attachObjectToBone(const String &boneName, MovableObject *pMovable, const Quaternion &offsetOrientation, const Vector3 &offsetPosition)
@@ -964,7 +1030,7 @@ namespace Ogre {
         {
             mMesh->prepareForShadowVolume();
             // reset frame last updated to force update of buffers
-            mFrameAnimationLastUpdated = 0;
+            mFrameAnimationLastUpdated = std::numeric_limits<unsigned long>::max();
             // re-prepare buffers
             prepareTempBlendBuffers();
         }
@@ -1140,6 +1206,36 @@ namespace Ogre {
         return 0;
     }
     //-----------------------------------------------------------------------
+    void Entity::addSoftwareSkinningRequest(bool normalsAlso) 
+    {
+        mSoftwareSkinningRequests++;
+        if (normalsAlso) {
+            mSoftwareSkinningNormalsRequests++;
+        }
+        if(!mMesh->isPreparedForShadowVolumes())
+        {
+            mMesh->prepareForShadowVolume();
+            // re-prepare buffers
+            prepareTempBlendBuffers();
+        }
+    }
+    //-----------------------------------------------------------------------
+    void Entity::removeSoftwareSkinningRequest(bool normalsAlso)
+    {
+        if (mSoftwareSkinningRequests == 0 ||
+            (normalsAlso && mSoftwareSkinningNormalsRequests == 0)) 
+        {
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+                        "Attempt to remove nonexistant request.",
+                        "Entity::removeSoftwareSkinningRequest");
+        }
+        mSoftwareSkinningRequests--;
+        if (normalsAlso) {
+            mSoftwareSkinningNormalsRequests--;
+        }
+        // TODO: possibly undo "shadow volume" prep if no longer needed
+    }
+    //-----------------------------------------------------------------------
     void Entity::_notifyAttached(Node* parent, bool isTagPoint)
     {
         MovableObject::_notifyAttached(parent, isTagPoint);
@@ -1372,5 +1468,12 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
+	void Entity::refreshAvailableAnimationState(void)
+	{
+		if (hasSkeleton())
+		{
+			mSkeletonInstance->_refreshAnimationState(mAnimationState);
+		}
+	}
 
 }
