@@ -30,7 +30,6 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreVector4.h"
 #include "OgreAutoParamDataSource.h"
 #include "OgreLight.h"
-#include "OgreControllerManager.h"
 #include "OgreRoot.h"
 #include "OgreRenderSystem.h"
 #include "OgreRenderSystemCapabilities.h"
@@ -152,6 +151,11 @@ namespace Ogre
     {
         setConstant(index, vec.val, 1);
     }
+	//-----------------------------------------------------------------------------
+	void GpuProgramParameters::setConstant(size_t index, Real val)
+	{
+		setConstant(index, Vector4(val, 0.0f, 0.0f, 0.0f));
+	}
     //-----------------------------------------------------------------------------
     void GpuProgramParameters::setConstant(size_t index, const Vector3& vec)
     {
@@ -302,6 +306,9 @@ namespace Ogre
             case ACT_WORLD_MATRIX:
                 setConstant(i->index, source.getWorldMatrix());
                 break;
+            case ACT_TIME:
+               setConstant(i->index, Vector4(source.getTime() * i->fData, 0.f, 0.f, 0.f));
+               break;
            case ACT_TIME_0_X:
                setConstant(i->index, Vector4(source.getTime_0_X(i->fData), 0.f, 0.f, 0.f));
                break;
@@ -640,9 +647,7 @@ namespace Ogre
     //---------------------------------------------------------------------------
     void GpuProgramParameters::setConstantFromTime(size_t index, Real factor)
     {
-        // Create controller
-        ControllerManager::getSingleton().createGpuProgramTimerParam(this, index, factor);
-
+        setAutoConstantReal(index, ACT_TIME, factor);
     }
     //---------------------------------------------------------------------------
     void GpuProgramParameters::setNamedConstantFromTime(const String& name, Real factor)
@@ -821,6 +826,9 @@ namespace Ogre
         if (pRep == r.getPointer())
             return *this;
         release();
+		// lock & copy other mutex pointer
+		OGRE_LOCK_MUTEX(*r.OGRE_AUTO_MUTEX_NAME)
+		OGRE_COPY_AUTO_SHARED_MUTEX(r.OGRE_AUTO_MUTEX_NAME)
         pRep = r.getPointer();
         pUseCount = r.useCountPointer();
         if (pUseCount)
