@@ -40,8 +40,19 @@ namespace Ogre {
 	Material::Material(ResourceManager* creator, const String& name, ResourceHandle handle,
 		const String& group, bool isManual, ManualResourceLoader* loader)
 		:Resource(creator, name, handle, group, isManual, loader),
-		mCompilationRequired(true), mReceiveShadows(true), mTransparencyCastsShadows(false)
+         mReceiveShadows(true),
+         mTransparencyCastsShadows(false),
+         mCompilationRequired(true)
     {
+		// Override isManual, not applicable for Material (we always want to call loadImpl)
+		if(isManual)
+		{
+			mIsManual = false;
+			LogManager::getSingleton().logMessage("Material " + name + 
+				" was requested with isManual=true, but this is not applicable " 
+				"for materials; the flag has been reset to false");
+		}
+
 		mLodDistances.push_back(0.0f);
 
 		applyDefaults();
@@ -72,8 +83,9 @@ namespace Ogre {
 	    mHandle = rhs.mHandle;
         mSize = rhs.mSize;
         mReceiveShadows = rhs.mReceiveShadows;
+        mTransparencyCastsShadows = rhs.mTransparencyCastsShadows;
 
-
+        mIsLoaded = rhs.mIsLoaded;
 
         // Copy Techniques
         this->removeAllTechniques();
@@ -92,16 +104,15 @@ namespace Ogre {
             }
         }
 
+        // Fixup the best technique list guarantees no gaps inside
+        fixupBestTechniqueList();
+
 		// Also copy LOD information
-		mLodDistances.clear();
-		LodDistanceList::const_iterator lodi, lodiend;
-		lodiend = rhs.mLodDistances.end();
-		for (lodi = rhs.mLodDistances.begin(); lodi != lodiend; ++lodi)
-		{
-			mLodDistances.push_back(*lodi);
-		}
-        mCompilationRequired = rhs.mCompilationRequired; 
-        mIsLoaded = rhs.mIsLoaded;
+		mLodDistances = rhs.mLodDistances;
+        mCompilationRequired = rhs.mCompilationRequired;
+        // illumination passes are not compiled right away so
+        // mIsLoaded state should still be the same as the original material
+        assert(mIsLoaded == rhs.mIsLoaded);
 
 	    return *this;
     }
@@ -159,7 +170,6 @@ namespace Ogre {
 			newMat->mGroup = newGroup;
 		}
 		
-		newMat->mIsLoaded = this->mIsLoaded;
         // Correct the name & handle, they get copied too
         newMat->mName = newName;
         newMat->mHandle = newHandle;
@@ -176,12 +186,16 @@ namespace Ogre {
         ResourceHandle savedHandle = mat->mHandle;
         String savedName = mat->mName;
         String savedGroup = mat->mGroup;
+		ManualResourceLoader* savedLoader = mat->mLoader;
+		bool savedManual = mat->mIsManual;
         // Assign values from this
         *mat = *this;
         // Correct the name & handle, they get copied too
         mat->mName = savedName;
         mat->mHandle = savedHandle;
         mat->mGroup = savedGroup;
+		mat->mIsManual = savedManual;
+		mat->mLoader = savedLoader;
 
     }
     //-----------------------------------------------------------------------
@@ -195,11 +209,15 @@ namespace Ogre {
             String savedName = mName;
             String savedGroup = mGroup;
             ResourceHandle savedHandle = mHandle;
+			ManualResourceLoader *savedLoader = mLoader;
+			bool savedManual = mIsManual;
 			*this = *defaults;
             // restore name & handle
             mName = savedName;
             mHandle = savedHandle;
             mGroup = savedGroup;
+			mLoader = savedLoader;
+			mIsManual = savedManual;
 		}
         mCompilationRequired = true;
 
@@ -323,25 +341,10 @@ namespace Ogre {
 					BestTechniqueList::value_type((*i)->getLodIndex(), *i));
             }
         }
-		// Now iterate over the best technique list, looking for gaps and filling them in
-		// guarantees we've got a sequential list with entries in all indexes
-		BestTechniqueList::iterator bi, biend;
-		biend = mBestTechniqueList.end();
-		unsigned short lastIndex = 0;
-		Technique* lastTechnique = NULL;
-		for (bi = mBestTechniqueList.begin(); bi != biend; ++bi)
-		{
-			while (bi->first > lastIndex + 1)
-			{
-				if (!lastTechnique) // hmm, index 0 is missing, use the first one we have
-					lastTechnique = bi->second;
-				mBestTechniqueList[++lastIndex] = lastTechnique;
-			}
 
-			lastIndex = bi->first;
-			lastTechnique = bi->second;
+        // Fixup the best technique list guarantees no gaps inside
+        fixupBestTechniqueList();
 
-		}
         mCompilationRequired = false;
 
         // Did we find any?
@@ -353,6 +356,32 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
+    void Material::fixupBestTechniqueList(void)
+    {
+		BestTechniqueList::iterator bi, biend;
+		biend = mBestTechniqueList.end();
+
+		unsigned short lastIndex = 0;
+		Technique* lastTechnique = 0;
+
+		for (bi = mBestTechniqueList.begin(); bi != biend; ++bi)
+		{
+			if (lastIndex < bi->first)
+			{
+				if (!lastTechnique) // hmm, index 0 is missing, use the first one we have
+					lastTechnique = bi->second;
+
+				do
+                {
+					mBestTechniqueList.insert(BestTechniqueList::value_type(lastIndex, lastTechnique));
+				} while (++lastIndex < bi->first);
+			}
+
+			++lastIndex;
+			lastTechnique = bi->second;
+		}
+    }
+    //-----------------------------------------------------------------------
     void Material::setAmbient(Real red, Real green, Real blue)
     {
         Techniques::iterator i, iend;
@@ -569,8 +598,8 @@ namespace Ogre {
     void Material::_notifyNeedsRecompile(void)
     {
         mCompilationRequired = true;
-        // Also need to flag as unloaded to ensure we loaded any new items
-        mIsLoaded = false;
+        // Also need to unload to ensure we loaded any new items
+        unload();
     }
     // --------------------------------------------------------------------
     void Material::setLodLevels(const LodDistanceList& lodDistances)
