@@ -147,8 +147,14 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     SubMesh* Mesh::getSubMesh(unsigned short index) const
     {
-        SubMeshList::const_iterator i = mSubMeshList.begin();
-        return const_cast<SubMesh*>(i[index]);
+        if (index >= mSubMeshList.size())
+        {
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+                "Index out of bounds.",
+                "Mesh::getSubMesh");
+        }
+
+        return mSubMeshList[index];
     }
 	//-----------------------------------------------------------------------
 	void Mesh::load(void)
@@ -206,6 +212,13 @@ namespace Ogre {
         // Removes all LOD data
         removeLodLevels();
         mPreparedForShadowVolumes = false;
+
+        // Clear bone assignments
+        mBoneAssignments.clear();
+        mBoneAssignmentsOutOfDate = false;
+
+        // Removes reference to skeleton
+        setSkeletonName(StringUtil::BLANK);
     }
 
     //-----------------------------------------------------------------------
@@ -712,16 +725,27 @@ namespace Ogre {
         if (mIsLodManual && index > 0 && mMeshLodUsageList[index].manualMesh.isNull())
         {
             // Load the mesh now
-            mMeshLodUsageList[index].manualMesh = 
-                MeshManager::getSingleton().load(
-                    mMeshLodUsageList[index].manualName,
-                    mGroup);
-            // get the edge data, if required
-            if (!mMeshLodUsageList[index].edgeData)
-            {
-                mMeshLodUsageList[index].edgeData = 
-                    mMeshLodUsageList[index].manualMesh->getEdgeList(0);
-            }
+			try {
+				mMeshLodUsageList[index].manualMesh = 
+					MeshManager::getSingleton().load(
+						mMeshLodUsageList[index].manualName,
+						mGroup);
+				// get the edge data, if required
+				if (!mMeshLodUsageList[index].edgeData)
+				{
+					mMeshLodUsageList[index].edgeData = 
+						mMeshLodUsageList[index].manualMesh->getEdgeList(0);
+				}
+			}
+			catch (Exception& e)
+			{	
+				StringUtil::StrStreamType str;
+				str << "Error while loading manual LOD level " 
+					<< mMeshLodUsageList[index].manualName 
+					<< " - this LOD level will not be rendered. You can "
+					<< "ignore this error in offline mesh tools.";
+				LogManager::getSingleton().logMessage(str.str());
+			}
 
         }
         return mMeshLodUsageList[index];
@@ -958,7 +982,7 @@ namespace Ogre {
             unsigned char* pDest = static_cast<unsigned char*>(
                 newBuffer->lock(HardwareBuffer::HBL_DISCARD));
             size_t vertSize = origBuffer->getVertexSize();
-            for (int v = 0; v < vertexData->vertexCount; ++v)
+            for (size_t v = 0; v < vertexData->vertexCount; ++v)
             {
                 // Copy original vertex data
                 memcpy(pDest, pSrc, vertSize);
@@ -987,7 +1011,7 @@ namespace Ogre {
         }
 
 	    // our temp. buffers
-	    unsigned short	vertInd[3];
+	    uint32			vertInd[3];
 	    Vector3         vertPos[3];
         Real            u[3], v[3];
 	    // setup a new 3D texture coord-set buffer for every sub mesh
@@ -996,7 +1020,8 @@ namespace Ogre {
 	    for (int sm = 0; sm < nSubMesh; sm++)
 	    {
 		    // retrieve buffer pointers
-		    unsigned short	*pVIndices;	// the face indices buffer, read only
+		    uint16			*pVIndices16;	// the face indices buffer, read only
+		    uint32			*pVIndices32;	// the face indices buffer, read only
 		    float			*p2DTC;		// pointer to 2D tex.coords, read only
 		    float			*p3DTC;		// pointer to 3D tex.coords, write/read (discard)
 		    float			*pVPos;		// vertex position buffer, read only
@@ -1006,8 +1031,19 @@ namespace Ogre {
 		    // retrieve buffer pointers
 		    // first, indices
 		    IndexData *indexData = pSubMesh->indexData;
-		    HardwareIndexBufferSharedPtr buffIndex = indexData->indexBuffer ;
-		    pVIndices = (unsigned short*) buffIndex->lock(HardwareBuffer::HBL_READ_ONLY); 
+		    HardwareIndexBufferSharedPtr buffIndex = indexData->indexBuffer;
+			bool use32bit = false;
+			if (buffIndex->getType() == HardwareIndexBuffer::IT_32BIT)
+			{
+		    	pVIndices32 = static_cast<uint32*>(
+					buffIndex->lock(HardwareBuffer::HBL_READ_ONLY)); 
+				use32bit = true;
+			}
+			else
+			{
+		    	pVIndices16 = static_cast<uint16*>(
+					buffIndex->lock(HardwareBuffer::HBL_READ_ONLY)); 
+			}
 		    // then, vertices
 		    VertexData *usedVertexData ;
 		    if (pSubMesh->useSharedVertices) {
@@ -1095,7 +1131,14 @@ namespace Ogre {
 			    for (i = 0; i < 3; ++i)
 			    {
 				    // get indexes of vertices that form a polygon in the position buffer
-				    vertInd[i] = *pVIndices++;
+				    if (use32bit)
+					{
+						vertInd[i] = *pVIndices32++;
+					}
+					else
+					{
+						vertInd[i] = *pVIndices16++;
+					}
 				    // get the vertices positions from the position buffer
                     unsigned char* vBase = pPosBase + (posInc * vertInd[i]);
                     elemVPos->baseVertexPointerToElement(vBase, &pVPos);
@@ -1275,7 +1318,10 @@ namespace Ogre {
             {
                 // Delegate edge building to manual mesh
                 // It should have already built it's own edge list while loading
-                usage.edgeData = usage.manualMesh->getEdgeList(0);
+				if (!usage.manualMesh.isNull())
+				{
+					usage.edgeData = usage.manualMesh->getEdgeList(0);
+				}
             }
             else
             {
