@@ -58,7 +58,7 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    Node::Node(const String& name)
+	Node::Node(const String& name) : Renderable()
     {
         mName = name;
         mParent = 0;
@@ -76,8 +76,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Node::~Node()
     {
-    }    
-
+		removeAllChildren();
+		if(mParent)
+			mParent->removeChild(this); 
+	}    
     //-----------------------------------------------------------------------
     Node* Node::getParent(void) const
     {
@@ -223,6 +225,8 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Node::addChild(Node* child)
     {
+        assert(!child->mParent);
+        
         mChildren.insert(ChildNodeMap::value_type(child->getName(), child));
         child->setParent(this);
 
@@ -466,12 +470,24 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     const Vector3 & Node::_getDerivedScale(void) const
     {
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+            mNeedParentUpdate = false;
+        }
         return mDerivedScale;
     }
     //-----------------------------------------------------------------------
     void Node::removeAllChildren(void)
     {
+		ChildNodeMap::iterator i, iend;
+		iend = mChildren.end();
+		for (i = mChildren.begin(); i != iend; ++i)
+		{
+			i->second->setParent(0);
+		}
         mChildren.clear();
+		mChildrenToUpdate.clear();
     }
     //-----------------------------------------------------------------------
     void Node::setScale(const Vector3& scale)
@@ -701,6 +717,11 @@ namespace Ogre {
     {
         return ChildNodeIterator(mChildren.begin(), mChildren.end());
     }
+	//-----------------------------------------------------------------------
+	Node::ConstChildNodeIterator Node::getChildIterator(void) const
+	{
+		return ConstChildNodeIterator(mChildren.begin(), mChildren.end());
+	}
     //-----------------------------------------------------------------------
     void Node::_weightedTransform(Real weight, const Vector3& translate, 
        const Quaternion& rotate, const Vector3& scale)
