@@ -234,7 +234,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     size_t ParticleSystem::getParticleQuota(void) const
     {
-        return mParticlePool.size();
+        return mPoolSize;
     }
     //-----------------------------------------------------------------------
     void ParticleSystem::setParticleQuota(size_t size)
@@ -244,20 +244,8 @@ namespace Ogre {
 
         if( currSize < size )
         {
-            this->increasePool(size);
-
-            for( size_t i = currSize; i < size; ++i )
-            {
-                // Add new items to the queue
-                mFreeParticles.push_back( mParticlePool[i] );
-            }
-
+            // Will allocate particles on demand
             mPoolSize = size;
-            // Tell the renderer
-            if (mRenderer)
-            {
-                mRenderer->_notifyParticleQuota(size);
-            }
             
         }
     }
@@ -327,7 +315,7 @@ namespace Ogre {
 			    
         iEmitEnd = mEmitters.end();
         emitterCount = mEmitters.size();
-        emissionAllowed = getParticleQuota() - mActiveParticles.size();
+        emissionAllowed = mFreeParticles.size();
         totalRequested = 0;
 
         // Count up total requested emissions
@@ -763,11 +751,31 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void ParticleSystem::configureRenderer(void)
     {
+        // Actual allocate particles
+        size_t currSize = mParticlePool.size();
+        size_t size = mPoolSize;
+        if( currSize < size )
+        {
+            this->increasePool(size);
+
+            for( size_t i = currSize; i < size; ++i )
+            {
+                // Add new items to the queue
+                mFreeParticles.push_back( mParticlePool[i] );
+            }
+
+            // Tell the renderer, if already configured
+            if (mRenderer && mIsRendererConfigured)
+            {
+                mRenderer->_notifyParticleQuota(size);
+            }
+        }
+
         if (mRenderer && !mIsRendererConfigured)
         {
             mRenderer->_notifyParticleQuota(mParticlePool.size());
             if (mParentNode)
-                mRenderer->_notifyAttached(mParentNode);
+                mRenderer->_notifyAttached(mParentNode, mParentIsTagPoint);
             mRenderer->_notifyDefaultDimensions(mDefaultWidth, mDefaultHeight);
             createVisualParticles(0, mParticlePool.size());
             MaterialPtr mat = MaterialManager::getSingleton().load(
