@@ -193,12 +193,28 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
     void ParticleSystemManager::addTemplate(const String& name, ParticleSystem* sysTemplate)
     {
+		// check name
+		if (mSystemTemplates.find(name) != mSystemTemplates.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM, 
+				"ParticleSystem template with name '" + name + "' already exists.", 
+				"ParticleSystemManager::addTemplate");
+		}
+
         mSystemTemplates[name] = sysTemplate;
     }
     //-----------------------------------------------------------------------
     ParticleSystem* ParticleSystemManager::createTemplate(const String& name, 
         const String& resourceGroup)
     {
+		// check name
+		if (mSystemTemplates.find(name) != mSystemTemplates.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM, 
+				"ParticleSystem template with name '" + name + "' already exists.", 
+				"ParticleSystemManager::createTemplate");
+		}
+
         ParticleSystem* tpl = new ParticleSystem(name, resourceGroup);
         addTemplate(name, tpl);
         return tpl;
@@ -221,6 +237,13 @@ namespace Ogre {
     ParticleSystem* ParticleSystemManager::createSystem(const String& name, size_t quota, 
         const String& resourceGroup)
     {
+		// check name
+		if (mSystems.find(name) != mSystems.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM, 
+				"ParticleSystem with name '" + name + "' already exists.", 
+				"ParticleSystemManager::createSystem");
+		}
         ParticleSystem* sys = new ParticleSystem(name, resourceGroup);
         sys->setParticleQuota(quota);
         mSystems.insert( ParticleSystemMap::value_type( name, sys ) );
@@ -229,6 +252,13 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     ParticleSystem* ParticleSystemManager::createSystem(const String& name, const String& templateName)
     {
+		// check name
+		if (mSystems.find(name) != mSystems.end())
+		{
+			OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM, 
+				"ParticleSystem with name '" + name + "' already exists.", 
+				"ParticleSystemManager::createSystem");
+		}
         // Look up template
         ParticleSystem* pTemplate = getTemplate(templateName);
         if (!pTemplate)
