@@ -87,7 +87,7 @@ namespace Ogre {
 	
     //-----------------------------------------------------------------------------
 	Pass::Pass(Technique *parent, unsigned short index, const Pass& oth)
-        :mParent(parent), mIndex(index)
+        :mParent(parent), mIndex(index), mQueuedForDeletion(false)
     {
         *this = oth;
         mParent = parent;
@@ -171,9 +171,19 @@ namespace Ogre {
 		    mFragmentProgramUsage = NULL;
         }
 
-		// Copy texture units
-		removeAllTextureUnitStates();
 		TextureUnitStates::const_iterator i, iend;
+
+        // Clear texture units but doesn't notify need recompilation in the case
+        // we are cloning, The parent material will take care of this.
+        iend = mTextureUnitStates.end();
+        for (i = mTextureUnitStates.begin(); i != iend; ++i)
+        {
+            delete *i;
+        }
+
+        mTextureUnitStates.clear();
+
+		// Copy texture units
 		iend = oth.mTextureUnitStates.end();
 		for (i = oth.mTextureUnitStates.begin(); i != iend; ++i)
 		{
@@ -394,10 +404,18 @@ namespace Ogre {
     bool Pass::isTransparent(void) const
     {
 		// Transparent if any of the destination colour is taken into account
-	    if (mDestBlendFactor != SBF_ZERO)
-		    return true;
-	    else
+		if (mDestBlendFactor == SBF_ZERO && 
+			mSourceBlendFactor != SBF_DEST_COLOUR && 
+			mSourceBlendFactor != SBF_ONE_MINUS_DEST_COLOUR && 
+			mSourceBlendFactor != SBF_DEST_ALPHA && 
+			mSourceBlendFactor != SBF_ONE_MINUS_DEST_ALPHA)
+		{
 		    return false;
+		}
+	    else
+		{
+		    return true;
+		}
     }
     //-----------------------------------------------------------------------
     void Pass::setDepthCheckEnabled(bool enabled)
@@ -589,7 +607,7 @@ namespace Ogre {
 			// Set the new pass to fallback using scene blend
 			newPass->setSceneBlending(
 				(*i)->getColourBlendFallbackSrc(), (*i)->getColourBlendFallbackDest());
-			// Add all the other passes
+			// Add all the other texture unit states
 			for (; i != iend; ++i)
 			{
 				newPass->addTextureUnitState(*i);
