@@ -289,7 +289,8 @@ namespace Ogre {
             for (child = mChildren.begin(); child != childend; ++child)
             {
                 SceneNode* sceneChild = static_cast<SceneNode*>(child->second);
-                sceneChild->_findVisibleObjects(cam, queue, includeChildren, displayNodes);
+                sceneChild->_findVisibleObjects(cam, queue, includeChildren, 
+					displayNodes, onlyShadowCasters);
             }
         }
 
@@ -384,9 +385,12 @@ namespace Ogre {
     {
         ChildNodeMap::iterator i, iend;
         iend = mChildren.end();
-        for (i = mChildren.begin(); i != iend; ++i)
+        for (i = mChildren.begin(); i != iend;)
         {
             SceneNode* sn = static_cast<SceneNode*>(i->second);
+			// increment iterator before destroying (iterator invalidated by 
+			// SceneManager::destroySceneNode because it causes removal from parent)
+			++i;
             sn->removeAndDestroyAllChildren();
             sn->getCreator()->destroySceneNode(sn->getName());
         }
