@@ -520,6 +520,15 @@ namespace Ogre {
 					node->_getDerivedScale());
 			}
 		}
+		// Iterate through all the child-nodes
+		SceneNode::ConstChildNodeIterator nodei = node->getChildIterator();
+
+		while (nodei.hasMoreElements())
+		{
+			const SceneNode* node = static_cast<const SceneNode*>(nodei.getNext());
+			// Add this subnode and its children...
+			addSceneNode( node );
+		} 
 	}
 	//--------------------------------------------------------------------------
 	void StaticGeometry::build(void)
@@ -682,6 +691,14 @@ namespace Ogre {
 		}
 		mLodBucketList.clear();
 
+		for (ShadowRenderableList::iterator s = mShadowRenderables.begin();
+			s != mShadowRenderables.end(); ++s)
+		{
+			delete *s;
+		}
+		mShadowRenderables.clear();
+		delete mEdgeList;
+
 		// no need to delete queued meshes, these are managed in StaticGeometry
 
 	}
@@ -807,32 +824,38 @@ namespace Ogre {
 	//--------------------------------------------------------------------------
 	void StaticGeometry::Region::_notifyCurrentCamera(Camera* cam)
 	{
-		// Determine active lod
+		// Calculate squared view depth
 		Vector3 diff = cam->getDerivedPosition() - mCentre;
+		Real squaredDepth = diff.squaredLength();
+
+		// Determine whether to still render
+		Real renderingDist = mParent->getRenderingDistance();
+		if (renderingDist > 0)
+		{
+			// Max distance to still render
+			Real maxDist = renderingDist + mBoundingRadius;
+			if (squaredDepth > Math::Sqr(maxDist))
+			{
+				mBeyondFarDistance = true;
+				return;
+			}
+		}
+
+		mBeyondFarDistance = false;
+
 		// Distance from the edge of the bounding sphere
-		mCamDistanceSquared = diff.squaredLength() 
-			- mBoundingRadius*mBoundingRadius;
+		mCamDistanceSquared = squaredDepth - mBoundingRadius * mBoundingRadius;
 		// Clamp to 0
 		mCamDistanceSquared = std::max(static_cast<Real>(0.0), mCamDistanceSquared);
 
-
-		Real maxDist = mParent->getSquaredRenderingDistance();
-		if (maxDist && mCamDistanceSquared > maxDist)
+		// Determine active lod
+		mCurrentLod = mLodSquaredDistances.size() - 1;
+		for (ushort i = 0; i < mLodSquaredDistances.size(); ++i)
 		{
-			mBeyondFarDistance = true;
-		}
-		else
-		{
-			mBeyondFarDistance = false;
-
-			mCurrentLod = mLodSquaredDistances.size() - 1;
-			for (ushort i = 0; i < mLodSquaredDistances.size(); ++i)
+			if (mLodSquaredDistances[i] > mCamDistanceSquared)
 			{
-				if (mLodSquaredDistances[i] > mCamDistanceSquared)
-				{
-					mCurrentLod = i - 1;
-					break;
-				}
+				mCurrentLod = i - 1;
+				break;
 			}
 		}
 		
@@ -1301,7 +1324,7 @@ namespace Ogre {
 	StaticGeometry::GeometryBucket::GeometryBucket(MaterialBucket* parent,
 		const String& formatString, const VertexData* vData, 
 		const IndexData* iData)
-		: mParent(parent), mFormatString(formatString)
+		: Renderable(), mParent(parent), mFormatString(formatString)
 	{
 		// Clone the structure from the example
 		mVertexData = vData->clone(false);
