@@ -36,7 +36,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 namespace Ogre {
     //-----------------------------------------------------------------------------
     Technique::Technique(Material* parent)
-        : mParent(parent), mIsSupported(false), mLodIndex(0)
+        : mParent(parent), mIsSupported(false), mIlluminationPassesCompilationPhase(IPS_NOT_COMPILED), mLodIndex(0)
     {
         // See above, defaults to unsupported until examined
     }
@@ -138,8 +138,9 @@ namespace Ogre {
         // If we got this far, we're ok
         mIsSupported = true;
 
-        // Now compile for categorised illumination, incase we need it
-        _compileIlluminationPasses();
+        // Compile for categorised illumination on demand
+        clearIlluminationPasses();
+        mIlluminationPassesCompilationPhase = IPS_NOT_COMPILED;
 
     }
     //-----------------------------------------------------------------------------
@@ -198,8 +199,9 @@ namespace Ogre {
 			Pass* p = new Pass(this, (*i)->getIndex(), *(*i));
 			mPasses.push_back(p);
 		}
-        // recompile illumination passes
-        _compileIlluminationPasses();
+        // Compile for categorised illumination on demand
+        clearIlluminationPasses();
+        mIlluminationPassesCompilationPhase = IPS_NOT_COMPILED;
 		return *this;
     }
     //-----------------------------------------------------------------------------
@@ -258,7 +260,7 @@ namespace Ogre {
 		for (il = mIlluminationPasses.begin(); il != ilend; ++il)
 		{
 			if((*il)->pass != (*il)->originalPass)
-			(*il)->pass->_load();
+			    (*il)->pass->_load();
 		}
     }
     //-----------------------------------------------------------------------------
@@ -275,7 +277,8 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     bool Technique::isLoaded(void) const
     {
-        return mParent->isLoaded();
+        // Only supported technique will be loaded
+        return mParent->isLoaded() && mIsSupported;
     }
     //-----------------------------------------------------------------------
     void Technique::setAmbient(Real red, Real green, Real blue)
@@ -494,7 +497,11 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Technique::_notifyNeedsRecompile(void)
     {
-        mParent->_notifyNeedsRecompile();
+        // Disable require to recompile when splitting illumination passes
+        if (mIlluminationPassesCompilationPhase != IPS_COMPILE_DISABLED)
+        {
+            mParent->_notifyNeedsRecompile();
+        }
     }
     //-----------------------------------------------------------------------
     void Technique::setLodIndex(unsigned short index)
@@ -519,6 +526,8 @@ namespace Ogre {
         
         IlluminationStage iStage = IS_AMBIENT;
 
+        // Disable requirement to recompile during compile
+
         bool haveAmbient = false;
         while (i != iend)
         {
@@ -547,8 +556,7 @@ namespace Ogre {
                         p->getSelfIllumination() != ColourValue::Black)
                     {
                         // Copy existing pass
-                        Pass* newPass = new Pass(this, p->getIndex());
-                        *newPass = *p;
+                        Pass* newPass = new Pass(this, p->getIndex(), *p);
                         // Remove any texture units
                         newPass->removeAllTextureUnitStates();
                         // Remove any fragment program
@@ -616,8 +624,7 @@ namespace Ogre {
                         p->getSpecular() != ColourValue::Black))
                     {
                         // Copy existing pass
-                        Pass* newPass = new Pass(this, p->getIndex());
-                        *newPass = *p;
+                        Pass* newPass = new Pass(this, p->getIndex(), *p);
                         // remove texture units
                         newPass->removeAllTextureUnitStates();
                         // remove fragment programs
@@ -660,8 +667,7 @@ namespace Ogre {
                     else
                     {
                         // Copy the pass and tweak away the lighting parts
-                        Pass* newPass = new Pass(this, p->getIndex());
-                        *newPass = *p;
+                        Pass* newPass = new Pass(this, p->getIndex(), *p);
                         newPass->setAmbient(ColourValue::Black);
                         newPass->setDiffuse(ColourValue::Black);
                         newPass->setSpecular(ColourValue::Black);
@@ -708,6 +714,16 @@ namespace Ogre {
     const Technique::IlluminationPassIterator 
     Technique::getIlluminationPassIterator(void)
     {
+        if (mIlluminationPassesCompilationPhase == IPS_NOT_COMPILED)
+        {
+            // prevents parent->_notifyNeedsRecompile() call during compile
+            mIlluminationPassesCompilationPhase = IPS_COMPILE_DISABLED;
+            // Splitting the passes into illumination passes
+            _compileIlluminationPasses();
+            // Mark that illumination passes compilation finished
+            mIlluminationPassesCompilationPhase = IPS_COMPILED;
+        }
+
         return IlluminationPassIterator(mIlluminationPasses.begin(), 
             mIlluminationPasses.end());
     }
