@@ -128,27 +128,16 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     TextureUnitState::~TextureUnitState()
     {
-        // Destroy controllers
-        if (mAnimController)
-        {
-            ControllerManager::getSingleton().destroyController(mAnimController);
-        }
-        // Destroy effect controllers
-        for (EffectMap::iterator i = mEffects.begin(); i != mEffects.end(); ++i)
-        {
-            if (i->second.controller)
-            {
-                ControllerManager::getSingleton().destroyController(i->second.controller);
-            }
-
-        }
-        // Don't unload textures. may be used elsewhere
-
+        // Unload ensure all controllers destroyed
+        _unload();
     }
     //-----------------------------------------------------------------------
     TextureUnitState & TextureUnitState::operator = ( 
         const TextureUnitState &oth )
     {
+        assert(mAnimController == 0);
+        assert(mEffects.empty());
+
         // copy basic members (int's, real's)
         memcpy( this, &oth, (uchar *)(&oth.mFrames[0]) - (uchar *)(&oth) );
 
@@ -158,7 +147,19 @@ namespace Ogre {
 
         mEffects = oth.mEffects;
 
-        mParent->_dirtyHash();
+        // Can't sharing controllers with other TUS, reset to null to avoid potential bug.
+        for (EffectMap::iterator j = mEffects.begin(); j != mEffects.end(); ++j)
+        {
+            j->second.controller = 0;
+        }
+
+        // Load immediately if Material loaded
+        if (isLoaded())
+        {
+            _load();
+            // Tell parent to recalculate hash
+            mParent->_dirtyHash();
+        }
 
         return *this;
     }
@@ -444,6 +445,12 @@ namespace Ogre {
             EffectMap::iterator i = mEffects.find(effect.type);
             if (i != mEffects.end())
             {
+                // Destroy old effect controller if exist
+                if (i->second.controller)
+                {
+                    ControllerManager::getSingleton().destroyController(i->second.controller);
+                }
+
                 mEffects.erase(i);
             }
         }
@@ -576,7 +583,7 @@ namespace Ogre {
         mRecalcTexMatrix = true;
     }
     //-----------------------------------------------------------------------
-    const Matrix4& TextureUnitState::getTextureTransform()
+    const Matrix4& TextureUnitState::getTextureTransform() const
     {
         if (mRecalcTexMatrix)
             recalcTextureMatrix();
@@ -584,7 +591,7 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    void TextureUnitState::recalcTextureMatrix()
+    void TextureUnitState::recalcTextureMatrix() const
     {
         // Assumption: 2D texture coords
         Matrix3 xform, rot;
@@ -700,6 +707,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void TextureUnitState::_load(void)
     {
+        // Unload first
+        _unload();
+
         // Load textures
         for (unsigned int i = 0; i < mNumFrames; ++i)
         {
@@ -736,13 +746,14 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void TextureUnitState::createAnimController(void)
     {
+        assert(mAnimController == 0);
         mAnimController = ControllerManager::getSingleton().createTextureAnimator(this, mAnimDuration);
 
     }
     //-----------------------------------------------------------------------
     void TextureUnitState::createEffectController(TextureEffect& effect)
     {
-        effect.controller = 0;
+        assert(effect.controller == 0);
         ControllerManager& cMgr = ControllerManager::getSingleton();
         switch (effect.type)
         {
@@ -885,7 +896,24 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
     void TextureUnitState::_unload(void)
     {
-        // TODO
+        // Destroy animation controller
+        if (mAnimController)
+        {
+            ControllerManager::getSingleton().destroyController(mAnimController);
+            mAnimController = 0;
+        }
+
+        // Destroy effect controllers
+        for (EffectMap::iterator i = mEffects.begin(); i != mEffects.end(); ++i)
+        {
+            if (i->second.controller)
+            {
+                ControllerManager::getSingleton().destroyController(i->second.controller);
+                i->second.controller = 0;
+            }
+        }
+
+        // Don't unload textures. may be used elsewhere
     }
     //-----------------------------------------------------------------------------
     bool TextureUnitState::isLoaded(void)
