@@ -522,6 +522,14 @@ SceneNode * OctreeSceneManager::createSceneNode( void )
 
 SceneNode * OctreeSceneManager::createSceneNode( const String &name )
 {
+    // Check name not used
+    if (mSceneNodes.find(name) != mSceneNodes.end())
+    {
+        OGRE_EXCEPT(
+            Exception::ERR_DUPLICATE_ITEM,
+            "A scene node with the name " + name + " already exists",
+            "OctreeSceneManager::createSceneNode" );
+    }
     OctreeNode * on = new OctreeNode( this, name );
     mSceneNodes[ on->getName() ] = on;
     return on;
@@ -1020,6 +1028,10 @@ void OctreeSceneManager::resize( const AxisAlignedBox &box )
     mOctree = new Octree( 0 );
     mOctree->mBox = box;
 
+	const Vector3 min = box.getMinimum();
+	const Vector3 max = box.getMaximum();
+	mOctree->mHalfSize = ( max - min ) * 0.5f;
+
     it = nodes.begin();
 
     while ( it != nodes.end() )
