@@ -62,7 +62,7 @@ namespace Ogre
 
     //-----------------------------------------------------------------------
     TerrainRenderable::TerrainRenderable(const String& name)
-        : mTerrain(0), mName(name),  mDeltaBuffers(0), mPositionBuffer(0)
+        : Renderable(), mTerrain(0), mName(name),  mDeltaBuffers(0), mPositionBuffer(0)
     {
         mForcedRenderLevel = -1;
         mLastNextLevel = -1;
@@ -208,11 +208,11 @@ namespace Ogre
                 *pSysPos++ = *pPos++ = height; // y
                 *pSysPos++ = *pPos++ = ( float ) j * msOptions->scale.z; //z
 
-                *pTex0++ = ( float ) i / ( float ) msOptions->pageSize;
-                *pTex0++ = ( float ) ( float ) j / ( float ) msOptions->pageSize;
+                *pTex0++ = ( float ) i / ( float ) ( msOptions->pageSize - 1 );
+                *pTex0++ = ( float ) j / ( float ) ( msOptions->pageSize - 1 );
 
-                *pTex1++ = ( ( float ) i / ( float ) msOptions->tileSize ) * msOptions->detailTile;
-                *pTex1++ = ( ( float ) ( float ) j / ( float ) msOptions->tileSize ) * msOptions->detailTile;
+                *pTex1++ = ( ( float ) i / ( float ) ( msOptions->tileSize - 1 ) ) * msOptions->detailTile;
+                *pTex1++ = ( ( float ) j / ( float ) ( msOptions->tileSize - 1 ) ) * msOptions->detailTile;
 
                 if ( height < min )
                     min = ( Real ) height;
@@ -249,8 +249,6 @@ namespace Ogre
         Real C = _calculateCFactor();
 
         _calculateMinLevelDist2( C );
-        if (msOptions->lit)
-            _calculateNormals();
 
     }
     //-----------------------------------------------------------------------
@@ -289,6 +287,7 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void TerrainRenderable::_calculateNormals()
     {
+
         Vector3 norm;
 
         assert (msOptions->lit && "No normals present");
