@@ -436,6 +436,9 @@ namespace Ogre
             }
         }
 
+		// If we're not paging, load immediate for convenience
+		if ( mActivePageSource && !mPagingEnabled )
+			mActivePageSource->requestPage(0,0);
 
 
     }
@@ -536,6 +539,8 @@ namespace Ogre
         else
         {
             // Single page
+            if (mTerrainPages.empty() || mTerrainPages[0].empty())
+                return 0;
             return mTerrainPages[0][0];
         }
     }
