@@ -52,15 +52,15 @@ namespace Ogre
 
     const Matrix4 PROJECTIONCLIPSPACE2DTOIMAGESPACE_PERSPECTIVE(
         0.5,    0,  0, -0.5, 
-        0, -0.5,  0, -0.5, 
-        0,    0,  0,   1,
-        0,    0,  0,   1);
+          0, -0.5,  0, -0.5, 
+          0,    0,  0,   -1,
+          0,    0,  0,    1);
 
     const Matrix4 PROJECTIONCLIPSPACE2DTOIMAGESPACE_ORTHO(
-        -0.5,    0,  0, -0.5, 
-        0, 0.5,  0, -0.5, 
-        0,    0,  0,   1,
-        0,    0,  0,   1);
+        -0.5,   0,  0, -0.5, 
+           0, 0.5,  0, -0.5, 
+           0,   0,  0,   -1,
+           0,   0,  0,    1);
 
 	//---------------------------------------------------------------------
 	D3D9RenderSystem::D3D9RenderSystem( HINSTANCE hInstance )
@@ -112,6 +112,10 @@ namespace Ogre
 
         mCurrentLights = 0;
 
+		// Enumerate events
+		mEventNames.push_back("DeviceLost");
+		mEventNames.push_back("DeviceRestored");
+
 
 		OgreUnguard();
 	}
@@ -293,7 +297,7 @@ namespace Ogre
 			it->second.currentValue = value;
 		else
 		{
-            str.clear();
+            str.str(StringUtil::BLANK);
             str << "Option named '" << name << "' does not exist.";
 			OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS, str.str(), "D3D9RenderSystem::setConfigOption" );
 		}
@@ -735,9 +739,13 @@ namespace Ogre
 			// GeForce4 Ti (and presumably GeForce3) does not
 			// render infinite projection properly, even though it does in GL
             // So exclude all cards prior to the FX range from doing infinite
-            D3DADAPTER_IDENTIFIER9 adapterID = mActiveD3DDriver->getAdapterIdentifier();
+            const D3DADAPTER_IDENTIFIER9& adapterID = mActiveD3DDriver->getAdapterIdentifier();
 			if (adapterID.VendorId != 0x10DE || // not nVidia
-                adapterID.DeviceId >= 0x0301) // or GeForce FX or above
+				!((adapterID.DeviceId >= 0x200 && adapterID.DeviceId <= 0x20F) || //gf3
+				  (adapterID.DeviceId >= 0x250 && adapterID.DeviceId <= 0x25F) || //gf4ti
+				  (adapterID.DeviceId >= 0x280 && adapterID.DeviceId <= 0x28F) || //gf4ti
+				  (adapterID.DeviceId >= 0x170 && adapterID.DeviceId <= 0x18F) || //gf4 go
+				  (adapterID.DeviceId >= 0x280 && adapterID.DeviceId <= 0x28F)))  //gf4ti go
 			{
 				mCapabilities->setCapability(RSC_INFINITE_FAR_PLANE);
 			}
@@ -756,7 +764,11 @@ namespace Ogre
 		// We always support rendertextures bigger than the frame buffer
         mCapabilities->setCapability(RSC_HWRENDER_TO_TEXTURE);
 
-        mCapabilities->log(LogManager::getSingleton().getDefaultLog());
+		Log* defaultLog = LogManager::getSingleton().getDefaultLog();
+		if (defaultLog)
+		{
+	        mCapabilities->log(defaultLog);
+		}
     }
     //---------------------------------------------------------------------
     void D3D9RenderSystem::convertVertexShaderCaps(void)
@@ -829,6 +841,9 @@ namespace Ogre
         ushort major, minor;
         major = static_cast<ushort>((mCaps.PixelShaderVersion & 0x0000FF00) >> 8);
         minor = static_cast<ushort>(mCaps.PixelShaderVersion & 0x000000FF);
+
+		bool ps2x = false;
+
         switch (major)
         {
         case 1:
@@ -847,7 +862,6 @@ namespace Ogre
                 mCapabilities->setMaxFragmentProgramVersion("ps_1_4");
                 break;
             }
-            break;
             // no boolean params allowed
             mCapabilities->setFragmentProgramConstantBoolCount(0);
             // no integer params allowed
@@ -856,10 +870,25 @@ namespace Ogre
             // NB in ps_1_x these are actually stored as fixed point values,
             // but they are entered as floats
             mCapabilities->setFragmentProgramConstantFloatCount(8);
+            break;
         case 2:
-            if (minor > 0)
-            {
-                mCapabilities->setMaxFragmentProgramVersion("ps_2_x");
+			if ((mCaps.PS20Caps.Caps & D3DPS20CAPS_NODEPENDENTREADLIMIT) &&
+				(mCaps.PS20Caps.Caps & D3DPS20CAPS_NOTEXINSTRUCTIONLIMIT))
+			{
+	            if ((mCaps.PS20Caps.Caps & D3DPS20CAPS_ARBITRARYSWIZZLE) &&
+					(mCaps.PS20Caps.Caps & D3DPS20CAPS_GRADIENTINSTRUCTIONS) &&
+					(mCaps.PS20Caps.Caps & D3DPS20CAPS_PREDICATION))
+	            {
+					// ps_2_a is more powerful than ps_2_b? odd
+		            mCapabilities->setMaxFragmentProgramVersion("ps_2_a");
+				}
+				else
+				{
+					mCapabilities->setMaxFragmentProgramVersion("ps_2_b");
+				}
+				// ps_2_x general
+				ps2x = true;
+
                 // 16 boolean params allowed
                 mCapabilities->setFragmentProgramConstantBoolCount(16);
                 // 16 integer params allowed, 4D
@@ -908,7 +937,7 @@ namespace Ogre
 
             mGpuProgramManager->_pushSyntaxCode("ps_3_0");
         case 2:
-            if (major > 2 || minor > 0)
+            if (ps2x)
                 mGpuProgramManager->_pushSyntaxCode("ps_2_x");
 
             mGpuProgramManager->_pushSyntaxCode("ps_2_0");
@@ -1059,22 +1088,6 @@ namespace Ogre
         }
 	}
 	//---------------------------------------------------------------------
-	void D3D9RenderSystem::ResizeRepositionWindow(HWND wich)
-	{
-		for (RenderTargetMap::iterator it = mRenderTargets.begin(); it != mRenderTargets.end(); ++it)
-		{
-			if (it->second->isActive())
-			{
-				D3D9RenderWindow *pWin32Window = (D3D9RenderWindow *)it->second;
-				if (pWin32Window->getWindowHandle() == wich)
-				{
-					pWin32Window->WindowMovedOrResized();
-					break;
-				}
-			}
-		}
-	}
-	//---------------------------------------------------------------------
 	void D3D9RenderSystem::setAmbientLight( float r, float g, float b )
 	{
 		HRESULT hr = __SetRenderState( D3DRS_AMBIENT, D3DCOLOR_COLORVALUE( r, g, b, 1.0f ) );
@@ -1221,52 +1234,30 @@ namespace Ogre
 		const ColourValue &specular, const ColourValue &emissive, Real shininess,
         TrackVertexColourType tracking )
 	{
-		// Remember last call
-		static ColourValue lastAmbient = ColourValue::Black;
-		static ColourValue lastDiffuse = ColourValue::Black;
-		static ColourValue lastSpecular = ColourValue::Black;
-		static ColourValue lastEmissive = ColourValue::Black;
-		static Real lastShininess = 0.0;
-        static TrackVertexColourType lastTracking = -1;
+		
+		D3DMATERIAL9 material;
+		material.Diffuse = D3DXCOLOR( diffuse.r, diffuse.g, diffuse.b, diffuse.a );
+		material.Ambient = D3DXCOLOR( ambient.r, ambient.g, ambient.b, ambient.a );
+		material.Specular = D3DXCOLOR( specular.r, specular.g, specular.b, specular.a );
+		material.Emissive = D3DXCOLOR( emissive.r, emissive.g, emissive.b, emissive.a );
+		material.Power = shininess;
 
-		// Only update if changed
-		if( ambient != lastAmbient || diffuse != lastDiffuse ||
-			specular != lastSpecular || emissive != lastEmissive ||
-			shininess != lastShininess)
-		{
-			D3DMATERIAL9 material;
-			material.Diffuse = D3DXCOLOR( diffuse.r, diffuse.g, diffuse.b, diffuse.a );
-			material.Ambient = D3DXCOLOR( ambient.r, ambient.g, ambient.b, ambient.a );
-			material.Specular = D3DXCOLOR( specular.r, specular.g, specular.b, specular.a );
-			material.Emissive = D3DXCOLOR( emissive.r, emissive.g, emissive.b, emissive.a );
-			material.Power = shininess;
+		HRESULT hr = mpD3DDevice->SetMaterial( &material );
+		if( FAILED( hr ) )
+			OGRE_EXCEPT( hr, "Error setting D3D material", "D3D9RenderSystem::_setSurfaceParams" );
 
-			HRESULT hr = mpD3DDevice->SetMaterial( &material );
-			if( FAILED( hr ) )
-				OGRE_EXCEPT( hr, "Error setting D3D material", "D3D9RenderSystem::_setSurfaceParams" );
 
-			// Remember the details
-			lastAmbient = ambient;
-			lastDiffuse = diffuse;
-			lastSpecular = specular;
-			lastEmissive = emissive;
-			lastShininess = shininess;
-		}
-        if(tracking != lastTracking) 
+		if(tracking != TVC_NONE) 
         {
-            if(tracking != TVC_NONE) 
-            {
-                mpD3DDevice->SetRenderState(D3DRS_COLORVERTEX, TRUE);
-                mpD3DDevice->SetRenderState(D3DRS_AMBIENTMATERIALSOURCE, (tracking&TVC_AMBIENT)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
-                mpD3DDevice->SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, (tracking&TVC_DIFFUSE)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
-                mpD3DDevice->SetRenderState(D3DRS_SPECULARMATERIALSOURCE, (tracking&TVC_SPECULAR)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
-                mpD3DDevice->SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE, (tracking&TVC_EMISSIVE)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
-            } 
-            else 
-            {
-                mpD3DDevice->SetRenderState(D3DRS_COLORVERTEX, FALSE);               
-            }
-            lastTracking = tracking;
+            __SetRenderState(D3DRS_COLORVERTEX, TRUE);
+            __SetRenderState(D3DRS_AMBIENTMATERIALSOURCE, (tracking&TVC_AMBIENT)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
+            __SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, (tracking&TVC_DIFFUSE)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
+            __SetRenderState(D3DRS_SPECULARMATERIALSOURCE, (tracking&TVC_SPECULAR)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
+            __SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE, (tracking&TVC_EMISSIVE)?D3DMCS_COLOR1:D3DMCS_MATERIAL);
+        } 
+        else 
+        {
+            __SetRenderState(D3DRS_COLORVERTEX, FALSE);               
         }
         
 	}
@@ -1381,26 +1372,22 @@ namespace Ogre
         // If this is a cubic reflection, we need to modify using the view matrix
         if (mTexStageDesc[stage].autoTexCoordType == TEXCALC_ENVIRONMENT_MAP_REFLECTION)
         {
-            D3DXMATRIX viewMatrix; 
-
-            // Get view matrix
-            mpD3DDevice->GetTransform(D3DTS_VIEW, &viewMatrix);
-            // Get transposed 3x3, ie since D3D is transposed just copy
+            // Get transposed 3x3
             // We want to transpose since that will invert an orthonormal matrix ie rotation
             Matrix4 ogreViewTransposed;
-            ogreViewTransposed[0][0] = viewMatrix.m[0][0];
-            ogreViewTransposed[0][1] = viewMatrix.m[0][1];
-            ogreViewTransposed[0][2] = viewMatrix.m[0][2];
+            ogreViewTransposed[0][0] = mViewMatrix[0][0];
+            ogreViewTransposed[0][1] = mViewMatrix[1][0];
+            ogreViewTransposed[0][2] = mViewMatrix[2][0];
             ogreViewTransposed[0][3] = 0.0f;
 
-            ogreViewTransposed[1][0] = viewMatrix.m[1][0];
-            ogreViewTransposed[1][1] = viewMatrix.m[1][1];
-            ogreViewTransposed[1][2] = viewMatrix.m[1][2];
+            ogreViewTransposed[1][0] = mViewMatrix[0][1];
+            ogreViewTransposed[1][1] = mViewMatrix[1][1];
+            ogreViewTransposed[1][2] = mViewMatrix[2][1];
             ogreViewTransposed[1][3] = 0.0f;
 
-            ogreViewTransposed[2][0] = viewMatrix.m[2][0];
-            ogreViewTransposed[2][1] = viewMatrix.m[2][1];
-            ogreViewTransposed[2][2] = viewMatrix.m[2][2];
+            ogreViewTransposed[2][0] = mViewMatrix[0][2];
+            ogreViewTransposed[2][1] = mViewMatrix[1][2];
+            ogreViewTransposed[2][2] = mViewMatrix[2][2];
             ogreViewTransposed[2][3] = 0.0f;
 
             ogreViewTransposed[3][0] = 0.0f;
@@ -1416,7 +1403,7 @@ namespace Ogre
             // Derive camera space to projector space transform
             // To do this, we need to undo the camera view matrix, then 
             // apply the projector view & projection matrices
-            newMat = mViewMatrix.inverse() * newMat;
+            newMat = mViewMatrix.inverse();
             newMat = mTexStageDesc[stage].frustum->getViewMatrix() * newMat;
             newMat = mTexStageDesc[stage].frustum->getProjectionMatrix() * newMat;
             if (mTexStageDesc[stage].frustum->getProjectionType() == PT_PERSPECTIVE)
@@ -1427,6 +1414,7 @@ namespace Ogre
             {
                 newMat = PROJECTIONCLIPSPACE2DTOIMAGESPACE_ORTHO * newMat;
             }
+            newMat = xForm * newMat;
 
         }
 
@@ -1434,7 +1422,8 @@ namespace Ogre
 		d3dMat = D3D9Mappings::makeD3DXMatrix(newMat);
 
 		// need this if texture is a cube map, to invert D3D's z coord
-		if (mTexStageDesc[stage].autoTexCoordType != TEXCALC_NONE)
+		if (mTexStageDesc[stage].autoTexCoordType != TEXCALC_NONE &&
+            mTexStageDesc[stage].autoTexCoordType != TEXCALC_PROJECTIVE_TEXTURE)
 		{
 			d3dMat._13 = -d3dMat._13;
 			d3dMat._23 = -d3dMat._23;
@@ -1672,6 +1661,8 @@ namespace Ogre
 	void D3D9RenderSystem::_setDepthBias(ushort bias)
 	{
 		float bias_float = static_cast<float>(-bias);
+		// scale down - certainly needed for nVidia
+		bias_float /= 250000.0f;
 		HRESULT hr = __SetRenderState(D3DRS_DEPTHBIAS, *(DWORD*)&bias_float);
 		if (FAILED(hr))
 			OGRE_EXCEPT(hr, "Error setting depth bias", "D3D9RenderSystem::_setDepthBias");
@@ -1950,6 +1941,11 @@ namespace Ogre
 			d3dvp.Y = vp->getActualTop();
 			d3dvp.Width = vp->getActualWidth();
 			d3dvp.Height = vp->getActualHeight();
+            if (target->requiresTextureFlipping())
+            {
+                // Convert "top-left" to "bottom-left"
+                d3dvp.Y = target->getHeight() - d3dvp.Height - d3dvp.Y;
+            }
 
 			// Z-values from 0.0 to 1.0 (TODO: standardise with OpenGL)
 			d3dvp.MinZ = 0.0f;
@@ -2470,6 +2466,18 @@ namespace Ogre
         Real bottom, Real top, Real nearPlane, Real farPlane, Matrix4& dest,
         bool forGpuProgram)
     {
+        // Correct position for off-axis projection matrix
+        if (!forGpuProgram)
+        {
+            Real offsetX = left + right;
+            Real offsetY = top + bottom;
+
+            left -= offsetX;
+            right -= offsetX;
+            top -= offsetY;
+            bottom -= offsetY;
+        }
+
         Real width = right - left;
         Real height = top - bottom;
         Real q, qn;
@@ -2607,7 +2615,7 @@ namespace Ogre
 		SecondaryWindowList::iterator sw;
 		for (sw = mSecondaryWindows.begin(); sw != mSecondaryWindows.end(); ++sw)
 		{
-			(*sw)->destroy();
+			(*sw)->destroyD3DResources();
 		}
 
 		// Reset the device, using the primary window presentation params
@@ -2638,7 +2646,12 @@ namespace Ogre
 		static_cast<D3D9HardwareBufferManager*>(mHardwareBufferManager)
 			->recreateDefaultPoolResources();
 
+		LogManager::getSingleton().logMessage("!!! Direct3D Device successfully restored.");
+
 		mDeviceLost = false;
+
+		fireEvent("DeviceRestored");
+
 	}
 	//---------------------------------------------------------------------
 	bool D3D9RenderSystem::isDeviceLost(void)
@@ -2648,9 +2661,12 @@ namespace Ogre
 	//---------------------------------------------------------------------
 	void D3D9RenderSystem::_notifyDeviceLost(void)
 	{
+		LogManager::getSingleton().logMessage("!!! Direct3D Device Lost!");
 		mDeviceLost = true;
 		// will have lost basic states
 		mBasicStatesInitialised = false;
+
+		fireEvent("DeviceLost");
 	}
 
 }
