@@ -48,7 +48,6 @@ namespace Ogre
 		mpVolumeTex(NULL),
         mpZBuff(NULL),
         mpTex(NULL),
-        mAutoGenMipmaps(false),
 		mDynamicTextures(false)
 	{
         _initDevice();
@@ -58,9 +57,15 @@ namespace Ogre
 	{
         // have to call this here reather than in Resource destructor
         // since calling virtual methods in base destructors causes crash
-        unload(); 
+		if (mIsLoaded)
+		{
+			unload(); 
+		}
+		else
+		{
+			freeInternalResources();
+		}
 	}
-
 	/****************************************************************************************/
 	void D3D9Texture::copyToTexture(TexturePtr& target)
 	{
@@ -171,6 +176,13 @@ namespace Ogre
 	/****************************************************************************************/
 	void D3D9Texture::loadImpl()
 	{
+		if (mUsage & TU_RENDERTARGET)
+		{
+			createInternalResources();
+			mIsLoaded = true;
+			return;
+		}
+		
 		// load based on tex.type
 		switch (this->getTextureType())
 		{
@@ -190,12 +202,7 @@ namespace Ogre
 
 	}
 	/****************************************************************************************/
-	void D3D9Texture::unloadImpl()
-	{
-		_freeResources();
-	}
-	/****************************************************************************************/
-	void D3D9Texture::_freeResources()
+	void D3D9Texture::freeInternalResourcesImpl()
 	{
 		SAFE_RELEASE(mpTex);
 		SAFE_RELEASE(mpNormTex);
@@ -224,7 +231,7 @@ namespace Ogre
             if (FAILED(hr))
 		    {
 			    OGRE_EXCEPT( hr, "Can't create cube texture", "D3D9Texture::_loadCubeTex" );
-			    this->_freeResources();
+			    this->freeInternalResources();
 		    }
 
             hr = mpCubeTex->QueryInterface(IID_IDirect3DBaseTexture9, (void **)&mpTex);
@@ -232,7 +239,7 @@ namespace Ogre
             if (FAILED(hr))
 		    {
 			    OGRE_EXCEPT( hr, "Can't get base texture", "D3D9Texture::_loadCubeTex" );
-			    this->_freeResources();
+			    this->freeInternalResources();
 		    }
 
             D3DSURFACE_DESC texDesc;
@@ -294,7 +301,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT( hr, "Can't get base texture", "D3D9Texture::_loadVolumeTex" );
-				this->_freeResources();
+				this->freeInternalResources();
 			}
 	
 			D3DVOLUME_DESC texDesc;
@@ -343,7 +350,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT( hr, "Can't get base texture", "D3D9Texture::_loadNormTex" );
-				this->_freeResources();
+				this->freeInternalResources();
 			}
 	
 			D3DSURFACE_DESC texDesc;
@@ -361,7 +368,7 @@ namespace Ogre
 		}
 	}
 	/****************************************************************************************/
-    void D3D9Texture::createInternalResources(void)
+    void D3D9Texture::createInternalResourcesImpl(void)
 	{
 		// If mSrcWidth and mSrcHeight are zero, the requested extents have probably been set
 		// through setWidth and setHeight, which set mWidth and mHeight. Take those values.
@@ -397,7 +404,7 @@ namespace Ogre
 			break;
 		default:
 			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Unknown texture type", "D3D9Texture::createInternalResources" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 	}
 	/****************************************************************************************/
@@ -409,10 +416,12 @@ namespace Ogre
 		// determine wich D3D9 pixel format we'll use
 		HRESULT hr;
 		D3DFORMAT d3dPF = this->_chooseD3DFormat();
+		// let's D3DX check the corrected pixel format
+		hr = D3DXCheckTextureRequirements(mpDev, NULL, NULL, NULL, 0, &d3dPF, mD3DPool);
 
 		// Use D3DX to help us create the texture, this way it can adjust any relevant sizes
 		DWORD usage = (mUsage & TU_RENDERTARGET) ? D3DUSAGE_RENDERTARGET : 0;
-		UINT numMips = mNumMipmaps + 1;
+		UINT numMips = mNumRequestedMipmaps + 1;
 		// Check dynamic textures
 		if (mUsage & TU_DYNAMIC)
 		{
@@ -425,22 +434,25 @@ namespace Ogre
 			{
 				mDynamicTextures = false;
 			}
-
 		}
 		// check if mip maps are supported on hardware
-		if ((mDevCaps.TextureCaps & D3DPTEXTURECAPS_MIPMAP) && mNumMipmaps > 0)
+		mMipmapsHardwareGenerated = false;
+		if (mDevCaps.TextureCaps & D3DPTEXTURECAPS_MIPMAP)
 		{
-			// use auto.gen. if available, and if desired
-            mAutoGenMipmaps = this->_canAutoGenMipmaps(usage, D3DRTYPE_TEXTURE, d3dPF);
-			if ((mUsage & TU_AUTOMIPMAP) && mAutoGenMipmaps)
+			if (mUsage & TU_AUTOMIPMAP && mNumRequestedMipmaps != 0)
 			{
-				usage |= D3DUSAGE_AUTOGENMIPMAP;
-				numMips = 0;
+				// use auto.gen. if available, and if desired
+				mMipmapsHardwareGenerated = this->_canAutoGenMipmaps(usage, D3DRTYPE_TEXTURE, d3dPF);
+				if (mMipmapsHardwareGenerated)
+				{
+					usage |= D3DUSAGE_AUTOGENMIPMAP;
+					numMips = 0;
+				}
 			}
 		}
 		else
 		{
-			// device don't support mip maps, or zero mipmaps requested
+			// no mip map support for this kind of textures :(
 			mNumMipmaps = 0;
 			numMips = 1;
 		}
@@ -458,7 +470,7 @@ namespace Ogre
 		// check result and except if failed
 		if (FAILED(hr))
 		{
-			this->_freeResources();
+			this->freeInternalResources();
 			OGRE_EXCEPT( hr, "Error creating texture", "D3D9Texture::_createNormTex" );
 		}
 		
@@ -467,7 +479,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			OGRE_EXCEPT( hr, "Can't get base texture", "D3D9Texture::_createNormTex" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		
 		// set final tex. attributes from tex. description
@@ -477,7 +489,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			OGRE_EXCEPT( hr, "Can't get texture description", "D3D9Texture::_createNormTex" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		this->_setFinalAttributes(desc.Width, desc.Height, 1, this->_getPF(desc.Format));
 		
@@ -486,7 +498,7 @@ namespace Ogre
 			this->_createDepthStencil();
 
 		// Set best filter type
-		if(mAutoGenMipmaps)
+		if(mMipmapsHardwareGenerated)
 		{
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
@@ -504,19 +516,38 @@ namespace Ogre
 		// determine wich D3D9 pixel format we'll use
 		HRESULT hr;
 		D3DFORMAT d3dPF = this->_chooseD3DFormat();
+		// let's D3DX check the corrected pixel format
+		hr = D3DXCheckCubeTextureRequirements(mpDev, NULL, NULL, 0, &d3dPF, mD3DPool);
 
 		// Use D3DX to help us create the texture, this way it can adjust any relevant sizes
 		DWORD usage = (mUsage & TU_RENDERTARGET) ? D3DUSAGE_RENDERTARGET : 0;
-		UINT numMips = mNumMipmaps + 1;
+		UINT numMips = mNumRequestedMipmaps + 1;
+		// Check dynamic textures
+		if (mUsage & TU_DYNAMIC)
+		{
+			if (_canUseDynamicTextures(usage, D3DRTYPE_CUBETEXTURE, d3dPF))
+			{
+				usage |= D3DUSAGE_DYNAMIC;
+				mDynamicTextures = true;
+			}
+			else
+			{
+				mDynamicTextures = false;
+			}
+		}
 		// check if mip map cube textures are supported
+		mMipmapsHardwareGenerated = false;
 		if (mDevCaps.TextureCaps & D3DPTEXTURECAPS_MIPCUBEMAP)
 		{
-			// use auto.gen. if available
-            mAutoGenMipmaps = this->_canAutoGenMipmaps(usage, D3DRTYPE_CUBETEXTURE, d3dPF);
-			if (mAutoGenMipmaps)
+			if (mUsage & TU_AUTOMIPMAP && mNumRequestedMipmaps != 0)
 			{
-				usage |= D3DUSAGE_AUTOGENMIPMAP;
-				numMips = 0;
+				// use auto.gen. if available
+				mMipmapsHardwareGenerated = this->_canAutoGenMipmaps(usage, D3DRTYPE_CUBETEXTURE, d3dPF);
+				if (mMipmapsHardwareGenerated)
+				{
+					usage |= D3DUSAGE_AUTOGENMIPMAP;
+					numMips = 0;
+				}
 			}
 		}
 		else
@@ -538,7 +569,7 @@ namespace Ogre
 		// check result and except if failed
 		if (FAILED(hr))
 		{
-			this->_freeResources();
+			this->freeInternalResources();
 			OGRE_EXCEPT( hr, "Error creating texture", "D3D9Texture::_createCubeTex" );
 		}
 
@@ -547,7 +578,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			OGRE_EXCEPT( hr, "Can't get base texture", "D3D9Texture::_createCubeTex" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		
 		// set final tex. attributes from tex. description
@@ -557,7 +588,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			OGRE_EXCEPT( hr, "Can't get texture description", "D3D9Texture::_createCubeTex" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		this->_setFinalAttributes(desc.Width, desc.Height, 1, this->_getPF(desc.Format));
 		
@@ -566,7 +597,7 @@ namespace Ogre
 			this->_createDepthStencil();
 
 		// Set best filter type
-		if(mAutoGenMipmaps)
+		if(mMipmapsHardwareGenerated)
 		{
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
@@ -584,19 +615,38 @@ namespace Ogre
 		// determine wich D3D9 pixel format we'll use
 		HRESULT hr;
 		D3DFORMAT d3dPF = this->_chooseD3DFormat();
+		// let's D3DX check the corrected pixel format
+		hr = D3DXCheckVolumeTextureRequirements(mpDev, NULL, NULL, NULL, NULL, 0, &d3dPF, mD3DPool);
 
 		// Use D3DX to help us create the texture, this way it can adjust any relevant sizes
 		DWORD usage = (mUsage & TU_RENDERTARGET) ? D3DUSAGE_RENDERTARGET : 0;
-		UINT numMips = mNumMipmaps + 1;
-		// check if mip map cube textures are supported
+		UINT numMips = mNumRequestedMipmaps + 1;
+		// Check dynamic textures
+		if (mUsage & TU_DYNAMIC)
+		{
+			if (_canUseDynamicTextures(usage, D3DRTYPE_VOLUMETEXTURE, d3dPF))
+			{
+				usage |= D3DUSAGE_DYNAMIC;
+				mDynamicTextures = true;
+			}
+			else
+			{
+				mDynamicTextures = false;
+			}
+		}
+		// check if mip map volume textures are supported
+		mMipmapsHardwareGenerated = false;
 		if (mDevCaps.TextureCaps & D3DPTEXTURECAPS_MIPVOLUMEMAP)
 		{
-			// use auto.gen. if available
-            mAutoGenMipmaps = this->_canAutoGenMipmaps(usage, D3DRTYPE_VOLUMETEXTURE, d3dPF);
-			if (mAutoGenMipmaps)
+			if (mUsage & TU_AUTOMIPMAP && mNumRequestedMipmaps != 0)
 			{
-				usage |= D3DUSAGE_AUTOGENMIPMAP;
-				numMips = 0;
+				// use auto.gen. if available
+				mMipmapsHardwareGenerated = this->_canAutoGenMipmaps(usage, D3DRTYPE_VOLUMETEXTURE, d3dPF);
+				if (mMipmapsHardwareGenerated)
+				{
+					usage |= D3DUSAGE_AUTOGENMIPMAP;
+					numMips = 0;
+				}
 			}
 		}
 		else
@@ -620,7 +670,7 @@ namespace Ogre
 		// check result and except if failed
 		if (FAILED(hr))
 		{
-			this->_freeResources();
+			this->freeInternalResources();
 			OGRE_EXCEPT( hr, "Error creating texture", "D3D9Texture::_createVolumeTex" );
 		}
 
@@ -629,7 +679,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			OGRE_EXCEPT( hr, "Can't get base texture", "D3D9Texture::_createVolumeTex" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		
 		// set final tex. attributes from tex. description
@@ -639,7 +689,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			OGRE_EXCEPT( hr, "Can't get texture description", "D3D9Texture::_createVolumeTex" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		this->_setFinalAttributes(desc.Width, desc.Height, desc.Depth, this->_getPF(desc.Format));
 		
@@ -648,7 +698,7 @@ namespace Ogre
 			this->_createDepthStencil();
 
 		// Set best filter type
-		if(mAutoGenMipmaps)
+		if(mMipmapsHardwareGenerated)
 		{
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
@@ -765,7 +815,7 @@ namespace Ogre
 			break;
 		default:
 			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Unknown texture type", "D3D9Texture::_setSrcAttributes" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 	}
 	/****************************************************************************************/
@@ -832,6 +882,12 @@ namespace Ogre
 		assert(mpDev);
 		assert(mpD3D);
 
+		// Hacky override - many (all?) cards seem to not be able to autogen on 
+		// textures which are not a power of two
+		// Can we even mipmap on 3D textures? Well
+		if ((mWidth & mWidth-1) || (mHeight & mHeight-1) || (mDepth & mDepth-1))
+			return false;
+
 		if (mDevCaps.Caps2 & D3DCAPS2_CANAUTOGENMIPMAP)
 		{
 			HRESULT hr;
@@ -875,7 +931,7 @@ namespace Ogre
 		{
 			String msg = DXGetErrorDescription9(hr);
 			OGRE_EXCEPT( hr, "Error GetDepthStencilSurface : " + msg, "D3D9Texture::_createDepthStencil" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		// get it's description
 		hr = pSrf->GetDesc(&srfDesc);
@@ -884,7 +940,7 @@ namespace Ogre
 			String msg = DXGetErrorDescription9(hr);
 			OGRE_EXCEPT( hr, "Error GetDesc : " + msg, "D3D9Texture::_createDepthStencil" );
 			SAFE_RELEASE(pSrf);
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 		// release the temp. surface
 		SAFE_RELEASE(pSrf);
@@ -905,7 +961,7 @@ namespace Ogre
 		{
 			String msg = DXGetErrorDescription9(hr);
 			OGRE_EXCEPT( hr, "Error CreateDepthStencilSurface : " + msg, "D3D9Texture::_createDepthStencil" );
-			this->_freeResources();
+			this->freeInternalResources();
 		}
 	}
 	
@@ -913,7 +969,7 @@ namespace Ogre
 	// Macro to hide ugly cast
 	#define GETLEVEL(face,mip) \
 	 	static_cast<D3D9HardwarePixelBuffer*>(mSurfaceList[face*(mNumMipmaps+1)+mip].get())
-	void D3D9Texture::_createSurfaceList(bool updateOldList)
+	void D3D9Texture::_createSurfaceList(void)
 	{
 		IDirect3DSurface9 *surface;
 		IDirect3DVolume9 *volume;
@@ -933,6 +989,7 @@ namespace Ogre
 			bufusage = HardwareBuffer::HBU_STATIC;
 		}
 
+		bool updateOldList = mSurfaceList.size() == (getNumFaces() * (mNumMipmaps + 1));
 		if(!updateOldList)
 		{
 			// Create new list of surfaces
@@ -1002,11 +1059,11 @@ namespace Ogre
 		};
 		
 		// Set autogeneration of mipmaps for each face of the texture, if it is enabled
-		if(mNumMipmaps>0 && (mUsage & TU_AUTOMIPMAP)) 
+		if(mNumRequestedMipmaps != 0 && (mUsage & TU_AUTOMIPMAP)) 
 		{
 			for(face=0; face<getNumFaces(); ++face)
 			{
-				GETLEVEL(face, 0)->_setMipmapping(true, mAutoGenMipmaps, mpTex);
+				GETLEVEL(face, 0)->_setMipmapping(true, mMipmapsHardwareGenerated, mpTex);
 			}
 		}
 	}
@@ -1020,7 +1077,7 @@ namespace Ogre
 		if(mipmap > mNumMipmaps)
 			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Mipmap index out of range",
 					"D3D9Texture::getBuffer");
-		int idx = face*(mNumMipmaps+1) + mipmap;
+		size_t idx = face*(mNumMipmaps+1) + mipmap;
 		assert(idx < mSurfaceList.size());
 		return mSurfaceList[idx];
 	}
@@ -1173,36 +1230,58 @@ namespace Ogre
 		}
 	}
 	/****************************************************************************************/
-	void D3D9Texture::releaseIfDefaultPool(void)
+	bool D3D9Texture::releaseIfDefaultPool(void)
 	{
 		if(mD3DPool == D3DPOOL_DEFAULT)
 		{
-			unload();
+			LogManager::getSingleton().logMessage(
+				"Releasing D3D9 default pool texture: " + mName);
+			// Just free any internal resources, don't call unload() here
+			// because we want the un-touched resource to keep its unloaded status
+			// after device reset.
+			freeInternalResources();
+			LogManager::getSingleton().logMessage(
+				"Released D3D9 default pool texture: " + mName);
+			return true;
 		}
+		return false;
 	}
 	/****************************************************************************************/
-	void D3D9Texture::recreateIfDefaultPool(LPDIRECT3DDEVICE9 pDev)
+	bool D3D9Texture::recreateIfDefaultPool(LPDIRECT3DDEVICE9 pDev)
 	{
+		bool ret = false;
 		if(mD3DPool == D3DPOOL_DEFAULT)
 		{
-			// There are 2 possible scenarios here:
-			// 1. This is a render target
-			// 2. This is a dynamic texture, which probably won't have a loader,
-			//    but if it does, we'll call it
-			if ((mUsage & TU_RENDERTARGET) || !mLoader)
+			ret = true;
+			LogManager::getSingleton().logMessage(
+				"Recreating D3D9 default pool texture: " + mName);
+			// We just want to create the texture resources if:
+			// 1. This is a render texture, or
+			// 2. This is a manual texture with no loader, or
+			// 3. This was an unloaded regular texture (preserve unloaded state)
+			if ((mIsManual && !mLoader) || (mUsage & TU_RENDERTARGET) || !mIsLoaded)
 			{
-				// render target, or dynamic texture with no loader
-				// just recreate underlying surfaces
+				// just recreate any internal resources
 				createInternalResources();
 			}
+			// Otherwise, this is a regular loaded texture, or a manual texture with a loader
 			else
 			{
-				// Dynamic texture with a loader, call it
-                load();
+				// The internal resources already freed, need unload/load here:
+				// 1. Make sure resource memory usage statistic correction.
+				// 2. Don't call unload() in releaseIfDefaultPool() because we want
+				//    the un-touched resource keep unload status after device reset.
+				unload();
+				// if manual, we need to recreate internal resources since load() won't do that
+				if (mIsManual)
+					createInternalResources();
+				load();
 			}
+			LogManager::getSingleton().logMessage(
+				"Recreated D3D9 default pool texture: " + mName);
 		}
-		// re-query the surface list anyway
-		_createSurfaceList(true);
+
+		return ret;
 
 	}
 
