@@ -144,7 +144,7 @@ namespace Ogre {
 	}
 
     GLRenderSystem::GLRenderSystem()
-      : mExternalWindowHandle(0), mDepthWrite(true), mHardwareBufferManager(0),
+      : mDepthWrite(true), mHardwareBufferManager(0),
         mGpuProgramManager(0)
     {
         size_t i;
@@ -168,7 +168,8 @@ namespace Ogre {
 
         for (i = 0; i < OGRE_MAX_TEXTURE_COORD_SETS; i++)
         {
-            mTextureCoordIndex[i] = 0;
+			// Dummy value
+            mTextureCoordIndex[i] = 99;
         }
 
         for (i = 0; i < OGRE_MAX_TEXTURE_LAYERS; i++)
@@ -325,8 +326,20 @@ namespace Ogre {
         {
             GLint units;
             glGetIntegerv( GL_MAX_TEXTURE_UNITS, &units );
+			mFixedFunctionTextureUnits = units;
 
+			if (mGLSupport->checkExtension("GL_ARB_fragment_program"))
+			{
+				// Also check GL_MAX_TEXTURE_IMAGE_UNITS_ARB since NV at least
+				// only increased this on the FX/6x00 series
+				GLint arbUnits;
+				glGetIntegerv( GL_MAX_TEXTURE_IMAGE_UNITS_ARB, &arbUnits );
+				if (arbUnits > units)
+					units = arbUnits;
+			}
+	    
             mCapabilities->setNumTextureUnits(units);
+
         }
         else
         {
@@ -396,7 +409,20 @@ namespace Ogre {
 
             mGpuProgramManager->_pushSyntaxCode("arbvp1");
             mGpuProgramManager->registerProgramFactory("arbvp1", createGLArbGpuProgram);
-        }
+			if (mGLSupport->checkExtension("GL_NV_vertex_program2_option"))
+			{
+				mCapabilities->setMaxVertexProgramVersion("vp30");
+				mGpuProgramManager->_pushSyntaxCode("vp30");
+				mGpuProgramManager->registerProgramFactory("vp30", createGLArbGpuProgram);
+			}
+
+			if (mGLSupport->checkExtension("GL_NV_vertex_program3"))
+			{
+				mCapabilities->setMaxVertexProgramVersion("vp40");
+				mGpuProgramManager->_pushSyntaxCode("vp40");
+				mGpuProgramManager->registerProgramFactory("vp40", createGLArbGpuProgram);
+			}
+		}
 
         if (mGLSupport->checkExtension("GL_NV_register_combiners2") &&
             mGLSupport->checkExtension("GL_NV_texture_shader"))
@@ -446,7 +472,20 @@ namespace Ogre {
 
             mGpuProgramManager->_pushSyntaxCode("arbfp1");
             mGpuProgramManager->registerProgramFactory("arbfp1", createGLArbGpuProgram);
-        }
+			if (mGLSupport->checkExtension("GL_NV_fragment_program_option"))
+			{
+				mCapabilities->setMaxFragmentProgramVersion("fp30");
+				mGpuProgramManager->_pushSyntaxCode("fp30");
+				mGpuProgramManager->registerProgramFactory("fp30", createGLArbGpuProgram);
+			}
+
+			if (mGLSupport->checkExtension("GL_NV_fragment_program2"))
+			{
+				mCapabilities->setMaxFragmentProgramVersion("fp40");
+				mGpuProgramManager->_pushSyntaxCode("fp40");
+				mGpuProgramManager->registerProgramFactory("fp40", createGLArbGpuProgram);
+			}        
+		}
 
 		// NFZ - check if GLSL is supported
 		if ( mGLSupport->checkExtension("GL_ARB_shading_language_100") &&
@@ -625,7 +664,11 @@ namespace Ogre {
 		glGetQueryObjectuivARB_ptr =
 			(GL_GetQueryObjectuivARB_Func)mGLSupport->getProcAddress("glGetQueryObjectuivARB");
 
-        mCapabilities->log(LogManager::getSingleton().getDefaultLog());
+		Log* defaultLog = LogManager::getSingleton().getDefaultLog();
+		if (defaultLog)
+		{
+	        mCapabilities->log(defaultLog);
+		}
         mGLInitialized = true;
     }
 
@@ -896,13 +939,10 @@ namespace Ogre {
         mViewMatrix = m;
 
         GLfloat mat[16];
-        makeGLMatrix(mat, mViewMatrix);
+		makeGLMatrix( mat, mViewMatrix * mWorldMatrix );
         glMatrixMode(GL_MODELVIEW);
         glLoadMatrixf(mat);
 
-        makeGLMatrix(mat, mWorldMatrix);
-        glMultMatrixf(mat);
-
         setGLClipPlanes();
     }
     //-----------------------------------------------------------------------------
@@ -1010,24 +1050,37 @@ namespace Ogre {
 
             if(lastTextureType != mTextureTypes[stage] && lastTextureType != 0)
             {
-                glDisable( lastTextureType );
+				if (stage < mFixedFunctionTextureUnits)
+				{
+	                glDisable( lastTextureType );
+				}
             }
 
-			glEnable( mTextureTypes[stage] );
+			if (stage < mFixedFunctionTextureUnits)
+			{
+				glEnable( mTextureTypes[stage] );
+			}
+
 			if(!tex.isNull())
 				glBindTexture( mTextureTypes[stage], tex->getGLID() );
+			else
+				glBindTexture( mTextureTypes[stage], static_cast<GLTextureManager*>(mTextureManager)->getWarningTextureID() );
         }
         else
         {
-            if (lastTextureType != 0)
-            {
-                glDisable( mTextureTypes[stage] );
-            }
-			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+			if (stage < mFixedFunctionTextureUnits)
+			{
+				if (lastTextureType != 0)
+				{
+					glDisable( mTextureTypes[stage] );
+				}
+				glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+			}
         }
 
         glActiveTextureARB_ptr( GL_TEXTURE0 );
-    }
+
+	}
 
     //-----------------------------------------------------------------------------
     void GLRenderSystem::_setTextureCoordSet(size_t stage, size_t index)
@@ -1038,7 +1091,14 @@ namespace Ogre {
     void GLRenderSystem::_setTextureCoordCalculation(size_t stage, TexCoordCalcMethod m, 
         const Frustum* frustum)
     {
-        GLfloat M[16];
+		if (stage >= mFixedFunctionTextureUnits)
+		{
+			// Can't do this
+			return;
+		}
+
+		
+		GLfloat M[16];
         Matrix4 projectionBias;
 
         // Default to no extra auto texture matrix
@@ -1051,7 +1111,7 @@ namespace Ogre {
 
         glActiveTextureARB_ptr( GL_TEXTURE0 + stage );
 
-        switch( m )
+		switch( m )
         {
         case TEXCALC_NONE:
             glDisable( GL_TEXTURE_GEN_S );
@@ -1152,10 +1212,7 @@ namespace Ogre {
             mUseAutoTextureMatrix = true;
 
             // Set scale and translation matrix for projective textures
-            projectionBias = Matrix4::ZERO;
-            projectionBias[0][0] = 0.5; projectionBias[1][1] = -0.5; 
-            projectionBias[2][2] = 1.0; projectionBias[0][3] = 0.5; 
-            projectionBias[1][3] = 0.5; projectionBias[3][3] = 1.0;
+            projectionBias = Matrix4::CLIPSPACE2DTOIMAGESPACE;
 
             projectionBias = projectionBias * frustum->getProjectionMatrix();
             projectionBias = projectionBias * frustum->getViewMatrix();
@@ -1171,9 +1228,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     void GLRenderSystem::_setTextureAddressingMode(size_t stage, TextureUnitState::TextureAddressingMode tam)
     {
-        GLint type;
+		GLint type;
         switch(tam)
         {
+        default:
         case TextureUnitState::TAM_WRAP:
             type = GL_REPEAT;
             break;
@@ -1194,7 +1252,13 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     void GLRenderSystem::_setTextureMatrix(size_t stage, const Matrix4& xform)
     {
-        GLfloat mat[16];
+		if (stage >= mFixedFunctionTextureUnits)
+		{
+			// Can't do this
+			return;
+		}
+
+		GLfloat mat[16];
         makeGLMatrix(mat, xform);
 
 		if(mTextureTypes[stage] != GL_TEXTURE_3D && 
@@ -1203,6 +1267,8 @@ namespace Ogre {
 			// Convert 3x3 rotation/translation matrix to 4x4
 			mat[12] = mat[8];
 			mat[13] = mat[9];
+            mat[8] = 0;
+            mat[9] = 0;
 		}
 //        mat[14] = mat[10];
 //        mat[15] = mat[11];
@@ -1220,18 +1286,13 @@ namespace Ogre {
         glActiveTextureARB_ptr(GL_TEXTURE0 + stage);
         glMatrixMode(GL_TEXTURE);
 
+        // Load this matrix in
+        glLoadMatrixf(mat);
+
         if (mUseAutoTextureMatrix)
         {
-            // Load auto matrix in
-            glLoadMatrixf(mAutoTextureMatrix);
-            // Concat new matrix
-            glMultMatrixf(mat);
-
-        }
-        else
-        {
-            // Just load this matrix
-            glLoadMatrixf(mat);
+            // Concat auto matrix
+            glMultMatrixf(mAutoTextureMatrix);
         }
 
         glMatrixMode(GL_MODELVIEW);
@@ -1271,15 +1332,28 @@ namespace Ogre {
     {
         GLint sourceBlend = getBlendMode(sourceFactor);
         GLint destBlend = getBlendMode(destFactor);
-        
-        glEnable(GL_BLEND);
-        glBlendFunc(sourceBlend, destBlend);
+        if(sourceFactor == SBF_ONE && destFactor == SBF_ZERO)
+        {
+            glDisable(GL_BLEND);
+        }
+        else
+        {
+            glEnable(GL_BLEND);
+            glBlendFunc(sourceBlend, destBlend);
+        }
     }
     //-----------------------------------------------------------------------------
     void GLRenderSystem::_setAlphaRejectSettings(CompareFunction func, unsigned char value)
     {
-        glEnable(GL_ALPHA_TEST);
-        glAlphaFunc(convertCompareFunction(func), value / 255.0f);
+        if(func == CMPF_ALWAYS_PASS)
+        {
+            glDisable(GL_ALPHA_TEST);
+        }
+        else
+        {
+            glEnable(GL_ALPHA_TEST);
+            glAlphaFunc(convertCompareFunction(func), value / 255.0f);
+        }
     }
     //-----------------------------------------------------------------------------
     void GLRenderSystem::_setViewport(Viewport *vp)
@@ -1298,8 +1372,12 @@ namespace Ogre {
               w = vp->getActualWidth();
               h = vp->getActualHeight();
               x = vp->getActualLeft();
-              y = target->getHeight() - vp->getActualTop() - h;
-  
+              y = vp->getActualTop();
+              if (!target->requiresTextureFlipping())
+              {
+                  // Convert "upper-left" corner to "lower-left"
+                  y = target->getHeight() - h - y;
+              }
               glViewport(x, y, w, h);
   
               // Configure the viewport clipping
@@ -1330,12 +1408,11 @@ namespace Ogre {
             OGRE_EXCEPT(999, "Cannot begin frame - no viewport selected.",
                 "GLRenderSystem::_beginFrame");
 
+        // Activate the viewport clipping
+        glEnable(GL_SCISSOR_TEST);
         // Clear the viewport if required
         if (mActiveViewport->getClearEveryFrame())
         {
-            // Activate the viewport clipping
-            glEnable(GL_SCISSOR_TEST);
-
             clearFrameBuffer(FBT_COLOUR | FBT_DEPTH, 
                 mActiveViewport->getBackgroundColour());
         }        
@@ -1824,7 +1901,13 @@ namespace Ogre {
 	//-----------------------------------------------------------------------------
     void GLRenderSystem::_setTextureBlendMode(size_t stage, const LayerBlendModeEx& bm)
     {       
-        // Check to see if blending is supported
+		if (stage >= mFixedFunctionTextureUnits)
+		{
+			// Can't do this
+			return;
+		}
+
+		// Check to see if blending is supported
         if(!mCapabilities->hasCapability(RSC_BLENDING))
             return;
 
@@ -2081,6 +2164,10 @@ namespace Ogre {
             {
                 pBufferData = static_cast<const GLDefaultHardwareVertexBuffer*>(vertexBuffer.get())->getDataPtr(elem->getOffset());
             }
+            if (op.vertexData->vertexStart)
+            {
+                pBufferData = static_cast<char*>(pBufferData) + op.vertexData->vertexStart * vertexBuffer->getVertexSize();
+            }
 
             unsigned int i = 0;
 
@@ -2117,7 +2204,7 @@ namespace Ogre {
                 break;
             case VES_TEXTURE_COORDINATES:
 
-                for (i = 0; i < mCapabilities->getNumTextureUnits(); i++)
+                for (i = 0; i < OGRE_MAX_TEXTURE_COORD_SETS; i++)
                 {
 					// Only set this texture unit's texcoord pointer if it
 					// is supposed to be using this element's index
@@ -2212,12 +2299,11 @@ namespace Ogre {
         }
         else
         {
-            glDrawArrays(primType, op.vertexData->vertexStart,
-                op.vertexData->vertexCount);
+            glDrawArrays(primType, 0, op.vertexData->vertexCount);
         }
 
         glDisableClientState( GL_VERTEX_ARRAY );
-        for (int i = 0; i < mCapabilities->getNumTextureUnits(); i++)
+        for (int i = 0; i < OGRE_MAX_TEXTURE_COORD_SETS; i++)
         {
             glClientActiveTextureARB_ptr(GL_TEXTURE0 + i);
             glDisableClientState( GL_TEXTURE_COORD_ARRAY );
@@ -2327,6 +2413,8 @@ namespace Ogre {
     void GLRenderSystem::setScissorTest(bool enabled, size_t left, 
         size_t top, size_t right, size_t bottom)
     {
+        // If request texture flipping, use "upper-left", otherwise use "lower-left"
+        bool flipping = mActiveRenderTarget->requiresTextureFlipping();
         //  GL measures from the bottom, not the top
         size_t targetHeight = mActiveRenderTarget->getHeight();
         // Calculate the "lower-left" corner of the viewport
@@ -2337,7 +2425,10 @@ namespace Ogre {
             glEnable(GL_SCISSOR_TEST);
             // NB GL uses width / height rather than right / bottom
             x = left;
-            y = targetHeight - bottom;
+            if (flipping)
+                y = top;
+            else
+                y = targetHeight - bottom;
             w = right - left;
             h = bottom - top;
             glScissor(x, y, w, h);
@@ -2349,7 +2440,10 @@ namespace Ogre {
             w = mActiveViewport->getActualWidth();
             h = mActiveViewport->getActualHeight();
             x = mActiveViewport->getActualLeft();
-            y = targetHeight - mActiveViewport->getActualTop() - h;
+            if (flipping)
+                y = mActiveViewport->getActualTop();
+            else
+                y = targetHeight - mActiveViewport->getActualTop() - h;
             glScissor(x, y, w, h);
         }
     }
@@ -2477,18 +2571,6 @@ namespace Ogre {
         return 0.0f;
     }
  	//---------------------------------------------------------------------
- 	void GLRenderSystem::resizeRepositionWindow(void* wich)
- 	{
- 		mGLSupport->resizeRepositionWindow(wich);
- 		for (RenderTargetMap::iterator it = mRenderTargets.begin(); it != mRenderTargets.end(); ++it)		
- 		{
- 			if (it->second->isActive())
- 			{
- 				mGLSupport->resizeReposition(it->second);
- 			}
- 		}
- 	}
-    //---------------------------------------------------------------------
     void GLRenderSystem::_applyObliqueDepthProjection(Matrix4& matrix, const Plane& plane, 
         bool forGpuProgram)
     {
@@ -2522,6 +2604,7 @@ namespace Ogre {
         glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
         glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);        
         glEnable(GL_COLOR_SUM);
+        glDisable(GL_DITHER);
 
         // Check for FSAA
         // Enable the extension if it was enabled by the GLSupport
@@ -2537,20 +2620,49 @@ namespace Ogre {
         }
     }
     //---------------------------------------------------------------------
+    void GLRenderSystem::_switchContext(GLContext *context)
+    {
+        // Unbind GPU programs and rebind to new context later, because
+        // scene manager treat render system as ONE 'context' ONLY, and it
+        // cached the GPU programs using state.
+        if (mCurrentVertexProgram)
+            mCurrentVertexProgram->unbindProgram();
+        if (mCurrentFragmentProgram)
+            mCurrentFragmentProgram->unbindProgram();
+
+        // It's ready to switching
+        mCurrentContext->endCurrent();
+        mCurrentContext = context;
+        mCurrentContext->setCurrent();
+        
+        // Check if the context has already done one-time initialisation
+        if(!mCurrentContext->getInitialized()) 
+        {
+            _oneTimeContextInitialization();
+            mCurrentContext->setInitialized();
+        }
+
+        // Rebind GPU programs to new context
+        if (mCurrentVertexProgram)
+            mCurrentVertexProgram->bindProgram();
+        if (mCurrentFragmentProgram)
+            mCurrentFragmentProgram->bindProgram();
+
+        // Must reset depth/colour write mask to according with user desired, otherwise,
+        // clearFrameBuffer would be wrong because the value we are recorded may be
+        // difference with the really state stored in GL context.
+        glDepthMask(mDepthWrite);
+        glColorMask(mColourWrite[0], mColourWrite[1], mColourWrite[2], mColourWrite[3]);
+
+    }
+    //---------------------------------------------------------------------
     void GLRenderSystem::_setRenderTarget(RenderTarget *target)
     {
         mActiveRenderTarget = target;
         // Switch context if different from current one
         ContextMap::iterator i = mContextMap.find(target);
         if(i != mContextMap.end() && mCurrentContext != i->second) {
-            mCurrentContext->endCurrent();
-            mCurrentContext = i->second;
-            // Check if the context has already done one-time initialisation
-            if(!mCurrentContext->getInitialized()) {
-               _oneTimeContextInitialization();
-               mCurrentContext->setInitialized();
-            }
-            mCurrentContext->setCurrent();
+            _switchContext(i->second);
         }
     }
     //---------------------------------------------------------------------
@@ -2567,9 +2679,7 @@ namespace Ogre {
             // remains active. When this is the main context being unregistered,
             // we set the main context to 0.
             if(mCurrentContext != mMainContext) {
-                mCurrentContext->endCurrent();
-                mCurrentContext = mMainContext;
-                mCurrentContext->setCurrent();
+                _switchContext(mMainContext);
             } else {
                 mMainContext = 0;
             }
