@@ -49,6 +49,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 
 #include "OgreXSIMeshExporter.h"
 #include "OgreXSISkeletonExporter.h"
+#include "OgreXSIMaterialExporter.h"
 #include "OgreLogManager.h"
 #include "OgreException.h"
 #include "OgreXSIHelper.h"
@@ -58,10 +59,11 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "OgreMeshManager.h"
 #include "OgreSkeletonManager.h"
 #include "OgreDefaultHardwareBufferManager.h"
+#include "OgreMaterialManager.h"
 
 using namespace XSI;
 
-#define OGRE_XSI_EXPORTER_VERSION L"1.0.1"
+#define OGRE_XSI_EXPORTER_VERSION L"1.0.5"
 
 /** This is the main file for the OGRE XSI plugin.
 The purpose of the methods in this file are as follows:
@@ -163,7 +165,6 @@ XSI::CStatus OgreMeshExportCommand_Init( const XSI::CRef& context )
 	ArgumentArray args = cmd.GetArguments();
 
     args.Add( L"objectName", L"" );
-    args.Add( L"exportMesh", L"true" );
 	args.Add( L"targetMeshFileName", L"c:/default.mesh" );
 	args.Add( L"calculateEdgeLists", L"true" );
     args.Add( L"calculateTangents", L"false" );
@@ -235,19 +236,30 @@ extern "C"
 */
 XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 {	
-	Application app;
-	CStatus st(CStatus::OK);
-	Property prop = app.GetActiveSceneRoot().GetProperties().GetItem(exportPropertyDialogName);
-	if (!prop.IsValid())
-	{
-		prop = app.GetActiveSceneRoot().AddProperty(exportPropertyDialogName);
-		prop.PutParameterValue(L"version", OGRE_XSI_EXPORTER_VERSION);
-	}
 	Ogre::LogManager logMgr;
 	logMgr.createLog("OgreXSIExporter.log", true);
 	CString msg(L"OGRE Exporter Version ");
 	msg += OGRE_XSI_EXPORTER_VERSION;
 	LogOgreAndXSI(msg);
+
+	Application app;
+	CStatus st(CStatus::OK);
+	Property prop = app.GetActiveSceneRoot().GetProperties().GetItem(exportPropertyDialogName);
+	if (prop.IsValid())
+	{
+		// Check version number
+		CString currVersion(prop.GetParameterValue(L"version"));
+		if (!currVersion.IsEqualNoCase(OGRE_XSI_EXPORTER_VERSION))
+		{
+			DeleteObj(exportPropertyDialogName);
+			prop.ResetObject();
+		}
+	}
+	if (!prop.IsValid())
+	{
+		prop = app.GetActiveSceneRoot().AddProperty(exportPropertyDialogName);
+		prop.PutParameterValue(L"version", CString(OGRE_XSI_EXPORTER_VERSION));
+	}
 	
 	try
 	{
@@ -309,16 +321,21 @@ XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 
 			param = prop.GetParameters().GetItem( L"exportSkeleton" );
 			bool exportSkeleton = param.GetValue();
+			param = prop.GetParameters().GetItem( L"exportMaterials" );
+			bool exportMaterials = param.GetValue();
+			param = prop.GetParameters().GetItem( L"copyTextures" );
+			bool copyTextures = param.GetValue();
 
-
+			// create singletons
 			Ogre::ResourceGroupManager rgm;
 			Ogre::MeshManager meshMgr;
-			Ogre::SkeletonManager skelMgr;;
+			Ogre::SkeletonManager skelMgr;
+			Ogre::MaterialManager matMgr;
 			Ogre::DefaultHardwareBufferManager hardwareBufMgr;
 
 			
 			// determine number of exportsteps
-			size_t numSteps = 3;
+			size_t numSteps = 3 + 20;
 			if (numlods > 0)
 				numSteps++;
 			if (edgeLists)
@@ -329,6 +346,10 @@ XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 				numSteps += 3;
 
 			Ogre::ProgressManager progressMgr(numSteps);
+			
+			// Any material prefix? We need that for mesh linking too
+			param = prop.GetParameters().GetItem( L"materialPrefix" );
+			Ogre::String materialPrefix = XSItoOgre(param.GetValue());
 
 			if (exportSkeleton)
 			{
@@ -350,8 +371,9 @@ XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 				}
 
 				param = prop.GetParameters().GetItem( L"animationList" );
-				GridData gd = param.GetValue();
+				GridData gd(param.GetValue());
 				Ogre::AnimationList selAnimList;
+				bool anyIKSample = false;
 				for (int a = 0; a < gd.GetRowCount(); ++a)
 				{
 					if (gd.GetCell(1, a) == true)
@@ -362,6 +384,18 @@ XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 						{
 							if (ai->animationName == name)
 							{
+								if (gd.GetCell(2, a))
+								{
+									// IK sample
+									ai->ikSample = true;
+									ai->ikSampleInterval = gd.GetCell(3, a);
+									anyIKSample = true;
+
+								}
+								else
+								{
+									ai->ikSample = false;
+								}
 								selAnimList.push_back(*ai);
 								break;
 							}
@@ -369,6 +403,25 @@ XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 					}
 				}
 
+				// Warn about effect of IK sampling
+				if (anyIKSample)
+				{
+					long btn;
+					CStatus ret = app.GetUIToolkit().MsgBox(
+						L"You have chosen to sample one or more of your "
+						L"animations (in order to convert IK or other "
+						L"constraint-based animation). \n\n This will require "
+						L"all animation which has not yet been stored in an "
+						L"action to be removed, and the mixer to be cleared. "
+						L"Is this OK?", 
+						siMsgYesNo,
+						L"Animation sampling required",
+						btn);
+					if (btn != 6)
+						return CStatus::Fail;
+					
+				}
+
 				// Truncate the skeleton filename to just the name (no path)
 				Ogre::String skelName = skeletonFileName;
 				int pos = skeletonFileName.find_last_of("\\");
@@ -385,7 +438,8 @@ XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 				// Do the mesh
 				Ogre::DeformerMap& deformers = 
 					meshExporter.exportMesh(meshFileName, mergeSubmeshes, 
-						exportChildren, edgeLists, tangents, lodData, skelName);
+						exportChildren, edgeLists, tangents, materialPrefix,
+						lodData, skelName);
 				// do the skeleton
 				skelExporter.exportSkeleton(skeletonFileName, deformers, fps, selAnimList);
 
@@ -396,12 +450,31 @@ XSI::CStatus OnOgreMeshExportMenu( XSI::CRef& in_ref )
 			{
 				// Just mesh
 				meshExporter.exportMesh(meshFileName, mergeSubmeshes, 
-					exportChildren, edgeLists, tangents, lodData);
+					exportChildren, edgeLists, tangents, materialPrefix, lodData);
 			}
 
 			
 			delete lodData;
 
+			// Do we want to export materials too?
+			if (exportMaterials)
+			{
+				param = prop.GetParameters().GetItem( L"targetMaterialFileName" );
+				Ogre::String materialFileName = XSItoOgre(param.GetValue());
+				
+				Ogre::XsiMaterialExporter matExporter;
+				try 
+				{
+					matExporter.exportMaterials(meshExporter.getMaterials(), 
+						meshExporter.getTextureProjectionMap(), 
+						materialFileName, copyTextures, materialPrefix);
+				}
+				catch (Ogre::Exception& e)
+				{
+					// ignore, non-fatal and will be in log
+				}
+			}
+
 		}
 
 	}
@@ -450,10 +523,6 @@ CStatus OgreMeshExportOptions_Define( const CRef & in_Ctx )
 		L"Collection of selected objects", L"", 
 		nullValue, param) ;	
 	prop.AddParameter(	
-        L"exportMesh",CValue::siBool, caps, 
-		L"Export Mesh", L"", 
-		CValue(true), param) ;	
-	prop.AddParameter(	
         L"targetMeshFileName",CValue::siString, caps, 
 		L"Mesh Filename", L"", 
 		nullValue, param) ;	
@@ -512,6 +581,23 @@ CStatus OgreMeshExportOptions_Define( const CRef & in_Ctx )
         L"Frames per second", L"", 
         CValue(24l), param) ;	
 	prop.AddGridParameter(L"animationList");	
+	prop.AddParameter(
+		L"exportMaterials", CValue::siBool, caps, 
+		L"Export Materials", L"", 
+		CValue(true), param);
+	prop.AddParameter(
+		L"copyTextures", CValue::siBool, caps, 
+		L"Copy Textures To Folder", L"", 
+		CValue(true), param);
+    prop.AddParameter(	
+        L"targetMaterialFileName",CValue::siString, caps, 
+        L"Material Filename", L"", 
+        nullValue, param) ;	
+	prop.AddParameter(	
+		L"materialPrefix",CValue::siString, caps, 
+		L"Material Prefix", L"", 
+		nullValue, param) ;	
+		
 
 
 	return CStatus::OK;	
@@ -533,48 +619,29 @@ CStatus OgreMeshExportOptions_DefineLayout( const CRef & in_Ctx )
 
 	oLayout.Clear() ;
 
-    // Object 
+	// Mesh tab
 	oLayout.AddTab(L"Basic");
-    oLayout.AddGroup(L"Object(s) to Export");
+    // Object 
 
-    item = oLayout.AddItem(L"objectName");
+	oLayout.AddGroup(L"Object(s) to export");
+	item = oLayout.AddItem(L"objectName");
     item.PutAttribute( siUINoLabel, true );
+	oLayout.EndGroup();
+	
 	/*
     item.PutWidthPercentage(80);
     item = oLayout.AddButton(L"Refresh", L"Refresh");
     item.PutWidthPercentage(1) ;
 	*/
 
-    oLayout.EndGroup();
-
-	// Mesh group
-    item = oLayout.AddGroup(L"Mesh");
-    item = oLayout.AddItem(L"exportMesh") ;
+	oLayout.AddGroup(L"Mesh");
     item = oLayout.AddItem(L"targetMeshFileName", L"Target", siControlFilePath);
 	item.PutAttribute( siUINoLabel, true );
 	item.PutAttribute( siUIFileFilter, L"OGRE Mesh format (*.mesh)|*.mesh|All Files (*.*)|*.*||" );
 	item = oLayout.AddItem(L"mergeSubmeshes") ;
 	item = oLayout.AddItem(L"exportChildren") ;
 
-	oLayout.EndGroup();
-	// Skeleton Group
-	item = oLayout.AddGroup(L"Skeleton");
 
-	item = oLayout.AddItem(L"exportSkeleton");
-	item = oLayout.AddItem(L"targetSkeletonFileName", L"Target", siControlFilePath);
-	item.PutAttribute( siUINoLabel, true );
-	item.PutAttribute( siUIFileFilter, L"OGRE Skeleton format (*.skeleton)|*.skeleton|All Files (*.*)|*.*||" );
-	item = oLayout.AddItem(L"fps");
-	item = oLayout.AddItem(L"animationList", L"Animations", siControlGrid);
-	item.PutAttribute(siUIGridColumnWidths, L"0:120:60");
-	item.PutAttribute(siUIGridHideRowHeader, true);
-
-	oLayout.EndGroup();
-
-
-	oLayout.AddTab(L"Advanced");
-
-	oLayout.AddGroup(L"Mesh options");
     item = oLayout.AddItem(L"calculateEdgeLists");
     item = oLayout.AddItem(L"calculateTangents");
 	oLayout.AddGroup(L"Level of Detail Reduction");
@@ -588,6 +655,34 @@ CStatus OgreMeshExportOptions_DefineLayout( const CRef & in_Ctx )
 	item = oLayout.AddEnumControl(L"lodQuota", vals, L"Quota", XSI::siControlCombo);
 	item = oLayout.AddItem(L"lodReduction");
 	oLayout.EndGroup();
+	oLayout.EndGroup();
+
+	oLayout.AddTab(L"Materials");
+	// Material Tab
+    item = oLayout.AddItem(L"exportMaterials") ;
+    item = oLayout.AddItem(L"targetMaterialFileName", L"Target", siControlFilePath) ;
+	item.PutAttribute( siUINoLabel, true );
+	item.PutAttribute( siUIFileFilter, L"OGRE Material script (*.material)|*.material|All Files (*.*)|*.*||" );
+	item = oLayout.AddItem(L"materialPrefix");
+    item = oLayout.AddItem(L"copyTextures");
+	
+	
+	// Skeleton Tab
+	oLayout.AddTab(L"Animation");
+
+	item = oLayout.AddItem(L"exportSkeleton");
+	item = oLayout.AddItem(L"targetSkeletonFileName", L"Target", siControlFilePath);
+	item.PutAttribute( siUINoLabel, true );
+	item.PutAttribute( siUIFileFilter, L"OGRE Skeleton format (*.skeleton)|*.skeleton|All Files (*.*)|*.*||" );
+	item = oLayout.AddItem(L"fps");
+	item = oLayout.AddItem(L"animationList", L"Animations", siControlGrid);
+	item.PutAttribute(siUIGridColumnWidths, L"0:120:60:90:130");
+	item.PutAttribute(siUIGridHideRowHeader, true);
+	// Make animatino name read-only
+	item.PutAttribute(siUIGridReadOnlyColumns, L"1:0:0:0");
+
+
+
 
 
 
@@ -635,6 +730,37 @@ bool hasSkeleton(Selection& sel, bool recurse)
 	return false;
 }
 
+bool isAnimationIK(XSI::ActionSource& source)
+{
+	// Iterate over the animation items, and return true if any of them
+	// are effectors. 
+	Application app;
+	CRefArray items = source.GetItems();
+	for (int i = 0; i < items.GetCount(); ++i)
+	{
+		XSI::AnimationSourceItem item = items[i];
+
+		// Check the target
+		Ogre::String target = XSItoOgre(item.GetTarget());
+		size_t firstDotPos = target.find_first_of(".");
+		if (firstDotPos != Ogre::String::npos)
+		{
+			Ogre::String targetName = target.substr(0, firstDotPos);
+			// Find object
+			X3DObject targObj = app.GetActiveSceneRoot().FindChild(
+				OgretoXSI(targetName), L"", CStringArray());
+			if (targObj.IsValid())
+			{
+				if (targObj.IsA(siChainEffectorID))
+					return true;
+			}
+		}
+	}
+
+	return false;
+
+
+}
 
 void findAnimations(XSI::Model& model, Ogre::AnimationList& animList)
 {
@@ -661,6 +787,9 @@ void findAnimations(XSI::Model& model, Ogre::AnimationList& animList)
 				anim.startFrame = -1;
 				anim.endFrame = -1;
 				anim.source = ActionSource(src);
+				// default to sampling
+				anim.ikSample = true;
+				anim.ikSampleInterval = 5.0f;
 				animList.push_back(anim);
 			}
 		}
@@ -686,6 +815,13 @@ void getAnimations(XSI::Model& root, Ogre::AnimationList& animList)
 }
 
 
+struct AnimSetting
+{
+	bool toexport;
+	bool ik;
+	double ikSampleInterval;
+};
+
 #ifdef unix
 extern "C" 
 #endif
@@ -756,22 +892,60 @@ CStatus OgreMeshExportOptions_PPGEvent( const CRef& io_Ctx )
 			param = prop.GetParameters().GetItem(L"animationList");
 			param.PutCapabilityFlag(siReadOnly, false);
 			// value of param is a griddata object
-			// initialise it with all animations
-			GridData gd = param.GetValue();
-			gd.PutColumnType(1, siColumnBool);
+			// initialise it with all animations but try to remember previous values
+			GridData gd(param.GetValue());
+			// Store the existing settings
+			std::map<Ogre::String,AnimSetting> rememberedAnimations;
+			int row;
+			for (row = 0; row < gd.GetRowCount(); ++row)
+			{
+				AnimSetting s;
+				s.toexport = gd.GetCell(1, row);
+				s.ik = gd.GetCell(2, row);
+				s.ikSampleInterval = gd.GetCell(3, row);
+				Ogre::String animName = XSItoOgre(gd.GetCell(0, row));
+				rememberedAnimations[animName] = s;
+			}
 
-			gd.PutColumnCount(2);
+			// Second & third column is check box			
+			gd.PutColumnType(1, siColumnBool);
+			gd.PutColumnType(2, siColumnBool);
+			// Name is not adjustable
+
+
+			gd.PutColumnCount(4);
 			gd.PutColumnLabel(0, L"Name");
 			gd.PutColumnLabel(1, L"Export?");
+			gd.PutColumnLabel(2, L"Sample?");
+			gd.PutColumnLabel(3, L"Sample Interval");
 
 
-			getAnimations(app.GetActiveSceneRoot(), animList);
+			XSI::Model appRoot(app.GetActiveSceneRoot());
+			getAnimations(appRoot, animList);
 			gd.PutRowCount(animList.size());
-			int row = 0;
+			row = 0;
 			for (Ogre::AnimationList::iterator a = animList.begin(); a != animList.end(); ++a, ++row)
 			{
 				gd.PutCell(0, row, OgretoXSI(a->animationName));
-				gd.PutCell(1, row, true);
+				// do we have a setting for this already?
+				std::map<Ogre::String, AnimSetting>::iterator ra = 
+					rememberedAnimations.find(a->animationName);
+
+				if (ra != rememberedAnimations.end())
+				{
+					AnimSetting& s = ra->second;
+
+					gd.PutCell(1, row, s.toexport);
+					gd.PutCell(2, row, s.ik);
+					gd.PutCell(3, row, s.ikSampleInterval);
+				}
+				else
+				{
+					// default to true
+					gd.PutCell(1, row, true);
+					gd.PutCell(2, row, a->ikSample);
+					gd.PutCell(3, row, a->ikSampleInterval);
+				}
 			}
 			
 			hasSkel = true;
@@ -812,13 +986,25 @@ CStatus OgreMeshExportOptions_PPGEvent( const CRef& io_Ctx )
         // Check paramName against parameter names, perform custom onChanged event
 		if (paramName == L"targetMeshFileName")
 		{
+			// Default skeleton name if blank
 			Ogre::String meshName = XSItoOgre(changed.GetValue());
-			if (hasSkel && Ogre::StringUtil::endsWith(meshName, "mesh"))
+			if (hasSkel && Ogre::StringUtil::endsWith(meshName, "mesh") && 
+				prop.GetParameterValue(L"targetSkeletonFileName") == L"")
 			{
 				Ogre::String skelName = meshName.substr(0, meshName.size() - 4) + "skeleton";
 				CString xsiSkelName = OgretoXSI(skelName);
 				prop.PutParameterValue(L"targetSkeletonFileName", xsiSkelName);
 			}
+			if (Ogre::StringUtil::endsWith(meshName, "mesh") && 
+				prop.GetParameterValue(L"targetMaterialFileName") == L"")
+			{
+				// default material script name if blank
+				Ogre::String matName = meshName.substr(0, meshName.size() - 4) + "material";
+				CString xsiMatName = OgretoXSI(matName);
+				prop.PutParameterValue(L"targetMaterialFileName", xsiMatName);
+			}
+
+			
 		}
 	}
 
