@@ -35,6 +35,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include <xsi_selection.h>
 #include <xsi_envelope.h>
 #include <xsi_time.h>
+#include <xsi_source.h>
 
 #include "OgreException.h"
 #include "OgreXSIHelper.h"
@@ -85,11 +86,13 @@ namespace Ogre {
     {
 		/// Tidy up
 		cleanupDeformerMap();
+		cleanupMaterialMap();
     }
     //-----------------------------------------------------------------------
 	DeformerMap& XsiMeshExporter::exportMesh(const String& fileName, 
 		bool mergeSubMeshes, bool exportChildren, 
-		bool edgeLists, bool tangents, LodData* lod, const String& skeletonName)
+		bool edgeLists, bool tangents, const String& materialPrefix, 
+		LodData* lod, const String& skeletonName)
     {
 
 		LogOgreAndXSI(L"** Begin OGRE Mesh Export **");
@@ -100,13 +103,22 @@ namespace Ogre {
         MeshPtr pMesh = MeshManager::getSingleton().createManual("XSIExport", 
 			ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
 
+		mMaterialPrefix = materialPrefix;
+
 		cleanupDeformerMap();
+		cleanupMaterialMap();
 
 		// Find all PolygonMesh objects
 		buildPolygonMeshList(exportChildren);
 		// progress report
 		ProgressManager::getSingleton().progress();
 
+		// notify of skeleton beforehand
+		if (!skeletonName.empty())
+		{
+			pMesh->setSkeletonName(skeletonName);
+		}
+
 		// write the data into a mesh
 		buildMesh(pMesh.getPointer(), mergeSubMeshes, !skeletonName.empty());
 
@@ -145,11 +157,6 @@ namespace Ogre {
 
         }
 
-		if (!skeletonName.empty())
-		{
-			pMesh->setSkeletonName(skeletonName);
-		}
-
         MeshSerializer serializer;
         serializer.exportMesh(pMesh.getPointer(), fileName);
 
@@ -162,6 +169,16 @@ namespace Ogre {
 
 		return mXsiDeformerMap;
     }
+	//-----------------------------------------------------------------------
+	MaterialMap& XsiMeshExporter::getMaterials(void)
+	{
+		return mXsiMaterialMap;
+	}
+	//-----------------------------------------------------------------------
+	TextureProjectionMap& XsiMeshExporter::getTextureProjectionMap(void)
+	{
+		return mTextureProjectionMap;
+	}
 	//-----------------------------------------------------------------------
 	void XsiMeshExporter::buildMesh(Mesh* pMesh, bool mergeSubmeshes, bool lookForBoneAssignments)
 	{
@@ -179,11 +196,17 @@ namespace Ogre {
 		    
 			Finally, we bake any remaining protosubmeshes into submeshes.
 		*/
+		// Calculate the number of progress updates each mesh must raise
+		float progPerMesh = 20.0f / (float)(mXsiPolygonMeshList.size());
+		float currProg = 0.0f;
 		for (PolygonMeshList::iterator pm = mXsiPolygonMeshList.begin();
 			pm != mXsiPolygonMeshList.end(); ++pm)
 		{
+			currProg += progPerMesh;
+			unsigned short progUpdates = (unsigned short)currProg;
+			currProg -= progUpdates;
 			// build contents of this polymesh into ProtoSubMesh(es)
-			processPolygonMesh(pMesh, *pm, lookForBoneAssignments);
+			processPolygonMesh(pMesh, *pm, lookForBoneAssignments, progUpdates);
 
 			if (!mergeSubmeshes)
 			{
@@ -259,11 +282,16 @@ namespace Ogre {
 	}
 	//-----------------------------------------------------------------------
 	void XsiMeshExporter::processPolygonMesh(Mesh* pMesh, PolygonMeshEntry* xsiMesh, 
-		bool lookForBoneAssignments)
+		bool lookForBoneAssignments, unsigned short progressUpdates)
 	{
 		// Pre-process the mesh
 		if (!preprocessPolygonMesh(xsiMesh))
+		{
+			while(progressUpdates--)
+				ProgressManager::getSingleton().progress();
+
 			return;
+		}
 		
         // Retrieve all the XSI relevant summary info
         CPointRefArray pointArray(xsiMesh->mesh.GetPoints());
@@ -282,6 +310,19 @@ namespace Ogre {
 		String str = msg.str();
 		LogOgreAndXSI(str);
 #endif
+		if (mCurrentTextureCoordDimensions.size() > OGRE_MAX_TEXTURE_COORD_SETS)
+		{
+			// too many texture coordinates!
+			StringUtil::StrStreamType str;
+			str << "PolygonMesh '" << XSItoOgre(xsiMesh->mesh.GetName()) 
+				<< "' has too many texture coordinate sets (" 
+				<< mCurrentTextureCoordDimensions.size()
+				<< "); the limit is " << OGRE_MAX_TEXTURE_COORD_SETS;
+
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, str.str(), 
+				"XsiMeshExporter::processPolygonMesh");
+
+		}
 		
 		// Save transforms
 		MATH::CTransformation xsiTransform = xsiMesh->obj.GetKinematics().GetGlobal().GetTransform();
@@ -292,7 +333,9 @@ namespace Ogre {
         Vector3 min, max;
         bool first = true;
 
-        // Iterate through all the triangles
+		float progPerTri = (float)progressUpdates / triArray.GetCount();
+		float prog = 0.0f;
+		// Iterate through all the triangles
         // There will often be less positions than normals and UVs
         // But TrianglePoint
         for (long t = 0; t < triArray.GetCount(); ++t)
@@ -381,7 +424,16 @@ namespace Ogre {
 					max.makeCeil(vertex.position);
 				}
             }
-        }
+		
+			// Progress
+			prog += progPerTri;
+			while (prog >= 1.0f)
+			{
+				ProgressManager::getSingleton().progress();
+				prog -= 1.0f;
+			}
+
+		}
 
 		// Merge bounds
 		AxisAlignedBox box;
@@ -454,6 +506,10 @@ namespace Ogre {
             ClusterProperty uvProp(uvClusterPropertiesRefArray[i]);
 			CClusterPropertyElementArray uvElements = uvProp.GetElements();
 
+			// Add this to an map from uv set name to index
+			String textureProjectionName = XSItoOgre(uvProp.GetName());
+			mTextureProjectionMap[textureProjectionName] = i;
+
 			// Now, each Element here is actually a CDoubleArray of the u,v,w values
 			// However it's not in order of samplers, we need to use the Array
 			// linked to from the Elements collection under the cluster (not the 
@@ -501,8 +557,12 @@ namespace Ogre {
 		 * materials in question, and define the PolygonCluster map
 		 */
 		// Main material (will never exist if not merging submeshes)
+		String materialName = mMaterialPrefix + 
+			XSItoOgre(xsiMesh->obj.GetMaterial().GetName());
+		registerMaterial(materialName, xsiMesh->obj.GetMaterial());
+		
 		mMainProtoMesh = createOrRetrieveProtoSubMesh(
-			XSItoOgre(xsiMesh->obj.GetMaterial().GetName()), 
+			materialName, 
 			XSItoOgre(xsiMesh->obj.GetName()),
 			mCurrentTextureCoordDimensions, 
 			mCurrentHasVertexColours);
@@ -519,8 +579,11 @@ namespace Ogre {
 			// Is the material different for this poly cluster?
 			if (cluster.GetMaterial() != xsiMesh->obj.GetMaterial())
 			{
+				String submatName = mMaterialPrefix + 
+					XSItoOgre(cluster.GetMaterial().GetName());
+				registerMaterial(submatName, cluster.GetMaterial());
 				ProtoSubMesh* ps = createOrRetrieveProtoSubMesh(
-					XSItoOgre(cluster.GetMaterial().GetName()),
+					submatName,
 					XSItoOgre(cluster.GetName()),
 					mCurrentTextureCoordDimensions, 
 					mCurrentHasVertexColours);
@@ -579,14 +642,14 @@ namespace Ogre {
 		{
 			Cluster cluster(clusterRefArray[i]);	
 
-			// Get mapping from cluster element index to geometry position index
-			CLongArray derefArray = cluster.GetElements().GetArray();
-
 			CRefArray envelopes = cluster.GetEnvelopes();
 			for (int e = 0; e < envelopes.GetCount(); ++e)
 			{
 				Envelope envelope(envelopes[e]);
 
+				// Get mapping from cluster element index to geometry position index
+				CLongArray derefArray = envelope.GetElements(CTime().GetTime()).GetArray();
+
 				CRefArray deformers = envelope.GetDeformers();
 				for (int d = 0; d < deformers.GetCount(); ++d)
 				{
@@ -634,34 +697,37 @@ namespace Ogre {
 								// adjust index based on merging
 								size_t adjIndex = positionIndex + poli->second;
 								// look up real index
+								// If it doesn't exist, it's probably on a seam
+								// between clusters and we can safely skip it
 								IndexRemap::iterator remi = ps->posIndexRemap.find(adjIndex);
-								assert (remi != ps->posIndexRemap.end()); // should never fail
-
-								size_t vertIndex = remi->second;
-								bool moreVerts = true;
-								// add UniqueVertex and clones
-								while (moreVerts)
+								if (remi != ps->posIndexRemap.end())
 								{
-									UniqueVertex& vertex = ps->uniqueVertices[vertIndex];
-									VertexBoneAssignment vba;
-									vba.boneIndex = deformerEntry->boneID;
-									vba.vertexIndex = vertIndex;
-									vba.weight = weight;
-									ps->boneAssignments.insert(
-										Mesh::VertexBoneAssignmentList::value_type(vertIndex, vba));
-									atLeastOneAssignment = true;
 
-									if (vertex.nextIndex == 0)
+									size_t vertIndex = remi->second;
+									bool moreVerts = true;
+									// add UniqueVertex and clones
+									while (moreVerts)
 									{
-										moreVerts = false;
-									}
-									else
-									{
-										vertIndex = vertex.nextIndex;
+										UniqueVertex& vertex = ps->uniqueVertices[vertIndex];
+										VertexBoneAssignment vba;
+										vba.boneIndex = deformerEntry->boneID;
+										vba.vertexIndex = vertIndex;
+										vba.weight = weight;
+										ps->boneAssignments.insert(
+											Mesh::VertexBoneAssignmentList::value_type(vertIndex, vba));
+										atLeastOneAssignment = true;
+
+										if (vertex.nextIndex == 0)
+										{
+											moreVerts = false;
+										}
+										else
+										{
+											vertIndex = vertex.nextIndex;
+										}
 									}
 								}
 
-
 							}
 						}
 
@@ -684,6 +750,7 @@ namespace Ogre {
 			}
 		}
 
+
 	}
 	//-----------------------------------------------------------------------
 	void XsiMeshExporter::exportProtoSubMeshes(Mesh* pMesh)
@@ -705,6 +772,9 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	void XsiMeshExporter::exportProtoSubMesh(Mesh* pMesh, ProtoSubMesh* proto)
 	{
+		// Skip protos which have ended up empty
+		if (proto->indices.empty())
+			return;
 
         SubMesh* sm = 0;
         if (proto->name.empty())
@@ -763,14 +833,12 @@ namespace Ogre {
         offset += VertexElement::getTypeSize(VET_FLOAT3);
         sm->vertexData->vertexDeclaration->addElement(buf, offset, VET_FLOAT3, VES_NORMAL);
         offset += VertexElement::getTypeSize(VET_FLOAT3);
-        // TODO - split vertex data here if animated
-        /*
+        // split vertex data here if animated
         if (pMesh->hasSkeleton())
         {
-            buf = 0;
+            buf++;
             offset = 0;
         }
-        */
 		// Optional vertex colour
         if(proto->hasVertexColours)
         {
@@ -902,6 +970,16 @@ namespace Ogre {
 		mXsiDeformerMap.clear();
 	}
 	//-----------------------------------------------------------------------
+	void XsiMeshExporter::cleanupMaterialMap(void)
+	{
+		for (MaterialMap::iterator d = mXsiMaterialMap.begin();
+			d != mXsiMaterialMap.end(); ++d)
+		{
+			delete d->second;
+		}
+		mXsiMaterialMap.clear();
+	}
+	//-----------------------------------------------------------------------
 	void XsiMeshExporter::deriveSamplerIndices(const Triangle& tri, 
 		const PolygonFace& face, size_t* samplerIndices)
 	{
@@ -1116,4 +1194,25 @@ namespace Ogre {
 			}
 		}
     }
+    //-----------------------------------------------------------------------
+	void XsiMeshExporter::registerMaterial(const String& name, 
+		XSI::Material mat)
+	{
+		// Check we have a real-time shader based material first
+		XSI::Parameter rtParam = mat.GetParameter(L"RealTime");
+		
+		if(rtParam.GetSource().IsValid() && 
+			rtParam.GetSource().IsA(XSI::siShaderID))
+		{
+			MaterialMap::iterator i = mXsiMaterialMap.find(name);
+			if (i == mXsiMaterialMap.end())
+			{
+				// Add this one to the list
+				MaterialEntry* matEntry = new MaterialEntry();
+				matEntry->name = name;
+				matEntry->xsiShader = XSI::Shader(rtParam.GetSource());
+				mXsiMaterialMap[name] = matEntry;
+			}
+		}
+	}
 }
