@@ -48,6 +48,11 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include <xsi_matrix4.h>
 #include <xsi_transformation.h>
 #include <xsi_vector3.h>
+#include <xsi_constraint.h>
+#include <xsi_track.h>
+#include <xsi_clip.h>
+#include <xsi_selection.h>
+#include <xsi_statickinematicstate.h>
 
 using namespace XSI;
 
@@ -77,18 +82,28 @@ namespace Ogre
 	{
 		LogOgreAndXSI(L"** Begin OGRE Skeleton Export **");
 
+		copyDeformerMap(deformers);
+
 		SkeletonPtr skeleton = SkeletonManager::getSingleton().create("export",
 			ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
 		// construct the hierarchy
 		buildBoneHierarchy(skeleton.get(), deformers, animList);
+
+		// pre-parse all animations for lengths
+		determineAnimationLengths(animList);
+
 		// progress report
 		ProgressManager::getSingleton().progress();
 
+		establishInitialTransforms(deformers);
+
 		// create animations 
 		createAnimations(skeleton.get(), deformers, framesPerSecond, animList);
 		// progress report
 		ProgressManager::getSingleton().progress();
 
+		// Optimise
+		skeleton->optimiseAllAnimations();
 
 		SkeletonSerializer ser;
 		ser.exportSkeleton(skeleton.get(), skeletonFileName);
@@ -97,9 +112,23 @@ namespace Ogre
 
 		LogOgreAndXSI(L"** OGRE Skeleton Export Complete **");
 
+		cleanup();
 
 	}
 	//-----------------------------------------------------------------------------
+	void XsiSkeletonExporter::copyDeformerMap(DeformerMap& deformers)
+	{
+		// Make lower-case version
+		// some XSI animations appear to like to use case insensitive references :(
+		for (DeformerMap::iterator i = deformers.begin(); i != deformers.end(); ++i)
+		{
+			DeformerEntry* deformer = i->second;
+			String name = XSItoOgre(deformer->obj.GetName());
+			StringUtil::toLowerCase(name);
+			mLowerCaseDeformerMap[name] = deformer;
+		}
+	}
+	//-----------------------------------------------------------------------------
 	void XsiSkeletonExporter::buildBoneHierarchy(Skeleton* pSkeleton, 
 		DeformerMap& deformers, AnimationList& animList)
 	{
@@ -150,16 +179,41 @@ namespace Ogre
 			// link to parent
 			if (!deformer->parentName.empty())
 			{
-				DeformerMap::iterator p = deformers.find(deformer->parentName);
-				assert (p != deformers.end() && "Parent not found");
+				DeformerEntry* parent = getDeformer(deformer->parentName, deformers);
+				assert (parent && "Parent not found");
 				assert (deformer->pBone && "Child bone not created");
-				assert(p->second->pBone && "Parent bone not created");
-				DeformerEntry* parent = p->second;
+				assert(parent->pBone && "Parent bone not created");
 				parent->pBone->addChild(deformer->pBone);
 
 			}
 		}
 
+	}
+	//-----------------------------------------------------------------------------
+	DeformerEntry* XsiSkeletonExporter::getDeformer(const String& name, 
+		DeformerMap& deformers)
+	{
+		// Look in case sensitive list first
+		DeformerMap::iterator i = deformers.find(name);
+		if (i == deformers.end())
+		{
+			String lcaseName = name;
+			StringUtil::toLowerCase(lcaseName);
+			i = mLowerCaseDeformerMap.find(lcaseName);
+			if (i == mLowerCaseDeformerMap.end())
+			{
+				return 0;
+			}
+			else
+			{
+				return i->second;
+			}
+		}
+		else
+		{
+			return i->second;
+		}
+
 	}
 	//-----------------------------------------------------------------------------
 	void XsiSkeletonExporter::linkBoneWithParent(DeformerEntry* child, 
@@ -193,9 +247,8 @@ namespace Ogre
 
 			String parentName = XSItoOgre(parent.GetName());
 			// Otherwise, check to see if the parent is in the deformer list
-			DeformerMap::iterator i = deformers.find(parentName);
-			DeformerEntry* parentDeformer = 0;
-			if (i == deformers.end())
+			DeformerEntry* parentDeformer = getDeformer(parentName, deformers);
+			if (!parentDeformer)
 			{
 				// not found, create entry for parent 
 				parentDeformer = new DeformerEntry(deformers.size(), parent);
@@ -204,10 +257,6 @@ namespace Ogre
 				LogOgreAndXSI(CString(L"Added ") + parent.GetName() + 
 					CString(L" as a parent of ") + child->obj.GetName() );
 			}
-			else
-			{
-				parentDeformer = i->second;
-			}
 
 			// Link child entry with parent (not bone yet)
 			// link child to parent by name
@@ -215,6 +264,8 @@ namespace Ogre
 			parentDeformer->childNames.push_back(childName);
 
 
+
+
 		}
 
 	}
@@ -300,10 +351,9 @@ namespace Ogre
 				String paramName = target.substr(lastDotPos+1, 
 					target.size() - lastDotPos - 1);
 				// locate deformer
-				DeformerMap::iterator di = deformers.find(targetName);
-				if (di != deformers.end())
+				DeformerEntry* deformer = getDeformer(targetName, deformers);
+				if (deformer)
 				{
-					DeformerEntry* deformer = di->second;
 					// determine parameter
 					std::map<String, int>::iterator pi = mXSITrackTypeNames.find(paramName);
 					if (pi != mXSITrackTypeNames.end())
@@ -323,10 +373,6 @@ namespace Ogre
 		for (AnimationList::iterator ai = animList.begin(); ai != animList.end(); ++ai)
 		{
 			AnimationEntry& animEntry = *ai;
-			// tease out all the animation source items
-			processActionSource(animEntry.source, deformers);
-
-			determineAnimationLength(animEntry);
 
 			float animLength = (float)(animEntry.endFrame - animEntry.startFrame) / fps;
 			StringUtil::StrStreamType str;
@@ -335,11 +381,27 @@ namespace Ogre
 			LogOgreAndXSI(str.str());
 			Animation* anim = pSkel->createAnimation(animEntry.animationName, animLength);
 
-			createAnimationTracks(anim, animEntry, deformers, fps);
+			if (animEntry.ikSample)
+			{
+				createAnimationTracksSampled(anim, animEntry, deformers, fps);
+			}
+			else
+			{
+				createAnimationTracksDirect(anim, animEntry, deformers, fps);
+			}
 			
 		}
 	}
 	//-----------------------------------------------------------------------------
+	void XsiSkeletonExporter::determineAnimationLengths(AnimationList& animList)
+	{
+		for (AnimationList::iterator ai = animList.begin(); ai != animList.end(); ++ai)
+		{
+			AnimationEntry& animEntry = *ai;
+			determineAnimationLength(animEntry);
+		}
+	}
+	//-----------------------------------------------------------------------------
 	void XsiSkeletonExporter::determineAnimationLength(AnimationEntry& animEntry)
 	{
 		bool first = true;
@@ -356,7 +418,7 @@ namespace Ogre
 			CRefArray keys = fcurve.GetKeys();
 			for (int k = 0; k < keys.GetCount(); ++k)
 			{
-				long currFrame = fcurve.GetKeyTime(k).GetTime();
+				long currFrame = (long)(fcurve.GetKeyTime(k).GetTime());
 				if (first)
 				{
 					animEntry.startFrame = currFrame;
@@ -398,7 +460,7 @@ namespace Ogre
 			CRefArray keys = fcurve.GetKeys();
 			for (int k = 0; k < keys.GetCount(); ++k)
 			{
-				long currFrame = fcurve.GetKeyTime(k).GetTime();
+				long currFrame = (long)(fcurve.GetKeyTime(k).GetTime());
 				if (first)
 				{
 					animEntry.startFrame = currFrame;
@@ -420,14 +482,156 @@ namespace Ogre
 				animEntry.frames.insert(currFrame);
 			}
 		}
+
 	
 
 
 	}
 	//-----------------------------------------------------------------------------
-	void XsiSkeletonExporter::createAnimationTracks(Animation* pAnim, 
+	void XsiSkeletonExporter::createAnimationTracksSampled(Animation* pAnim, 
 		AnimationEntry& animEntry, DeformerMap& deformers, float fps)
 	{
+		// Save the current selection
+		CString seltext(mXsiApp.GetSelection().GetAsText());
+
+		// Clear current animation
+		CValueArray args;
+		CValue dummy;
+		mXsiApp.ExecuteCommand(L"SelectAll", args, dummy);
+		mXsiApp.ExecuteCommand(L"RemoveAllAnimation", args, dummy);
+
+		// Reset selection
+		mXsiApp.GetSelection().SetAsText(seltext);
+
+
+		// Create all tracks first
+		std::vector<AnimationTrack*> deformerTracks;
+		deformerTracks.resize(deformers.size());
+		for (DeformerMap::iterator di = deformers.begin(); di != deformers.end(); ++di)
+		{
+			DeformerEntry* deformer = di->second;
+			AnimationTrack* track = pAnim->createTrack(deformer->boneID, deformer->pBone);
+			deformerTracks[deformer->boneID] = track;
+		}
+
+		Model model = placeAnimationInMixer(animEntry);
+
+		// Iterate over frames, keying as we go
+		long numFrames = animEntry.endFrame - animEntry.startFrame;
+		if (animEntry.ikSampleInterval == 0)
+		{
+			// Don't allow zero samplign frequency - infinite loop!
+			animEntry.ikSampleInterval = 5.0f;
+		}
+
+		for (long frame = 0; frame < numFrames; frame += animEntry.ikSampleInterval)
+		{
+			sampleAllBones(deformers, deformerTracks, frame, fps);
+
+		}
+		// sample final frame
+		sampleAllBones(deformers, deformerTracks, 
+			animEntry.endFrame - animEntry.startFrame, fps);
+
+		// remove the clip we added
+		Mixer mixer(model.GetMixer());
+		removeAllFromMixer(mixer);
+
+	}
+	//-----------------------------------------------------------------------------
+	void XsiSkeletonExporter::establishInitialTransforms(DeformerMap& deformers)
+	{
+		for (DeformerMap::iterator di = deformers.begin(); di != deformers.end(); ++di)
+		{
+			DeformerEntry* deformer = di->second;
+			if (deformer->pBone->getParent() == 0)
+			{
+				// Based on global
+				deformer->initialXform = 
+					deformer->obj.GetKinematics().GetGlobal().GetTransform();
+			}
+			else
+			{
+				// Based on local
+				deformer->initialXform = 
+					deformer->obj.GetKinematics().GetLocal().GetTransform();
+			}
+
+		}
+	}
+	//-----------------------------------------------------------------------------
+	void XsiSkeletonExporter::sampleAllBones(DeformerMap& deformers, 
+		std::vector<AnimationTrack*> deformerTracks, double frame, float fps)
+	{
+		CValueArray args;
+		CValue dummy;
+		args.Resize(2);
+		// set the playcontrol 
+		args[0] = L"PlayControl.Key";
+		args[1] = frame;
+		mXsiApp.ExecuteCommand(L"SetValue", args, dummy);
+		args[0] = L"PlayControl.Current";
+		mXsiApp.ExecuteCommand(L"SetValue", args, dummy);
+
+		// Refresh
+		mXsiApp.ExecuteCommand(L"Refresh", CValueArray(), dummy);
+		// Sample all bones
+		for (DeformerMap::iterator di = deformers.begin(); di != deformers.end(); ++di)
+		{
+			DeformerEntry* deformer = di->second;
+			AnimationTrack* track = deformerTracks[deformer->boneID];
+
+			double initposx, initposy, initposz;
+			deformer->initialXform.GetTranslationValues(initposx, initposy, initposz);
+			double initrotx, initroty, initrotz;
+			deformer->initialXform.GetRotation().GetXYZAngles(initrotx, initroty, initrotz);
+			double initsclx, initscly, initsclz;
+			deformer->initialXform.GetScalingValues(initsclx, initscly, initsclz);
+			XSI::MATH::CMatrix4 invTrans = deformer->initialXform.GetMatrix4();
+			invTrans.InvertInPlace();
+
+			XSI::MATH::CTransformation transformation;
+			if (deformer->pBone->getParent() == 0)
+			{
+				// Based on global
+				transformation = 
+					deformer->obj.GetKinematics().GetGlobal().GetTransform();
+			}
+			else
+			{
+				// Based on local
+				transformation = 
+					deformer->obj.GetKinematics().GetLocal().GetTransform();
+			}
+
+			// Make relative to initial
+			XSI::MATH::CMatrix4 transformationMatrix = transformation.GetMatrix4();
+			transformationMatrix.MulInPlace(invTrans);
+			transformation.SetMatrix4(transformationMatrix);
+
+			double posx, posy, posz;
+			transformation.GetTranslationValues(posx, posy, posz);
+
+			// create keyframe
+			KeyFrame* kf = track->createKeyFrame((float)frame / fps);
+			// not sure why inverted transform doesn't work for position, but it doesn't
+			// I thought XSI used same transform order as OGRE
+			kf->setTranslate(XSItoOgre(transformation.GetTranslation()));
+			//kf->setTranslate(Vector3(posx - initposx, posy - initposy, posz - initposz));
+			kf->setRotation(XSItoOgre(transformation.GetRotationQuaternion()));
+			kf->setScale(XSItoOgre(transformation.GetScaling()));
+
+		}
+
+	}
+	//-----------------------------------------------------------------------------
+	void XsiSkeletonExporter::createAnimationTracksDirect(Animation* pAnim, 
+		AnimationEntry& animEntry, DeformerMap& deformers, float fps)
+	{
+
+		// tease out all the animation source items
+		processActionSource(animEntry.source, deformers);
+
 		/* We have to iterate over the list of deformers, and create a track
 		 * for each one. Since XSI stores keys for all 9 components separately,
 		 * we need to bake OGRE keyframes (which include position, rotation and 
@@ -496,30 +700,30 @@ namespace Ogre
 				double sclz = deriveKeyFrameValue(deformer->xsiTrack[XTT_SCL_Z], *fi, initsclz);
 
 
-
-
+				// Build transformation relative to initial
 				XSI::MATH::CTransformation transformation;
-
-				if (deformer->pBone->getParent() != 0)
-				{
-					transformation.SetTranslationFromValues(posx, posy, posz);
-					transformation.SetRotationFromXYZAnglesValues(
-						XSI::MATH::DegreesToRadians(rotx),
-						XSI::MATH::DegreesToRadians(roty),
-						XSI::MATH::DegreesToRadians(rotz),
-						XSI::MATH::CRotation::RotationOrder::siXYZ);
-					XSI::MATH::CVector3 scaling(sclx, scly, sclz);
-					transformation.SetScaling(scaling);
+				
+				XSI::MATH::CVector3 scaling(sclx, scly, sclz);
+				transformation.SetScaling(scaling);
+				transformation.SetRotationFromXYZAnglesValues(
+					XSI::MATH::DegreesToRadians(rotx),
+					XSI::MATH::DegreesToRadians(roty),
+					XSI::MATH::DegreesToRadians(rotz),
+					XSI::MATH::CRotation::siXYZ);
+				transformation.SetTranslationFromValues(posx, posy, posz);
 
 
-					XSI::MATH::CMatrix4 transformationMatrix = transformation.GetMatrix4();
-					transformationMatrix.MulInPlace(invTrans);
-					transformation.SetMatrix4(transformationMatrix);
-				}
+				XSI::MATH::CMatrix4 transformationMatrix = transformation.GetMatrix4();
+				transformationMatrix.MulInPlace(invTrans);
+				transformation.SetMatrix4(transformationMatrix);
+
 
 				// create keyframe
-				KeyFrame* kf = track->createKeyFrame((float)(*fi - 1) / fps);
-				kf->setTranslate(XSItoOgre(transformation.GetTranslation()));
+				KeyFrame* kf = track->createKeyFrame((float)(*fi - animEntry.startFrame) / fps);
+				// not sure why inverted transform doesn't work for position, but it doesn't
+				// I thought XSI used same transform order as OGRE
+				//kf->setTranslate(XSItoOgre(transformation.GetTranslation()));
+				kf->setTranslate(Vector3(posx - initposx, posy - initposy, posz - initposz));
 				kf->setRotation(XSItoOgre(transformation.GetRotationQuaternion()));
 				kf->setScale(XSItoOgre(transformation.GetScaling()));
 
@@ -544,4 +748,101 @@ namespace Ogre
 			return defaultVal;
 		}
 	}
+	//-----------------------------------------------------------------------------
+	void XsiSkeletonExporter::cleanup(void)
+	{
+
+		mLowerCaseDeformerMap.clear();
+
+		CValueArray args;
+		CValue dummy;
+		args.Resize(1);
+
+		for (int i = 0; i < mIKSampledAnimations.GetCount(); ++i)
+		{
+			args[0] = mIKSampledAnimations[i];
+			mXsiApp.ExecuteCommand(L"DeleteObj", args, dummy);
+		}
+		mIKSampledAnimations.Clear();
+
+	}
+	//-----------------------------------------------------------------------------
+	void XsiSkeletonExporter::removeAllFromMixer(Mixer& mixer)
+	{
+		CRefArray tracks(mixer.GetTracks());
+		for (int t = 0; t < tracks.GetCount(); ++t)
+		{
+			Track track(tracks[t]);
+			CRefArray clips(track.GetClips());
+			for (int c = 0; c < clips.GetCount(); ++c)
+			{
+				Clip clip(clips[c]);
+				CValueArray args;
+				CValue dummy;
+				args.Add(clip.GetFullName());
+				mXsiApp.ExecuteCommand(L"DeleteObj", args, dummy);
+			}
+
+		}
+
+	}
+	//-----------------------------------------------------------------------------
+	XSI::Mixer XsiSkeletonExporter::getMixer(AnimationEntry& anim)
+	{
+		Model model(anim.source.GetModel());
+		if (!model.HasMixer())
+		{
+			model = mXsiApp.GetActiveSceneRoot();
+		}
+		return model.GetMixer();
+	}
+	//-----------------------------------------------------------------------------
+	XSI::Model XsiSkeletonExporter::placeAnimationInMixer(AnimationEntry& anim)
+	{
+		Mixer mixer(getMixer(anim));
+		Model model(mixer.GetModel()) ;
+
+		removeAllFromMixer(mixer);
+
+		// Clear all clips from the mixer
+
+		CValueArray args;
+		CValue dummy;
+
+		// Add the new clip to the mixer
+		CRefArray tracks(mixer.GetTracks());
+		if (tracks.GetCount() == 0)
+		{
+			// Must be at least one track
+			args.Resize(3);
+			args[0] = model.GetFullName();
+			args[1] = mixer.GetFullName();
+			args[2] = 0.0f;
+			mXsiApp.ExecuteCommand(L"AddTrack", args, dummy);
+			tracks = mixer.GetTracks();
+		}
+		Track instrack(tracks[0]);
+		args.Resize(5);
+		args[0] = model.GetFullName(); // target model
+		args[1] = anim.source.GetFullName(); // source
+		args[2] = L""; // compound clip
+		args[3] = instrack.GetFullName();
+		args[4] = 0.0f; // start frame
+		mXsiApp.ExecuteCommand(L"AddClip", args, dummy);
+
+
+		// Reset the animation to frame 0, such that following animations start with the initial bone transforms
+		args.Resize(2);
+		args[0] = L"PlayControl.Key";
+		args[1] = (long)0;
+		mXsiApp.ExecuteCommand(L"SetValue", args, dummy);
+		args[0] = L"PlayControl.Current";
+		args[1] = (long)0;
+		mXsiApp.ExecuteCommand(L"SetValue", args, dummy);
+		mXsiApp.ExecuteCommand(L"Refresh", CValueArray(), dummy);
+
+
+		return model;
+
+	}
 }
